# -*- coding: utf-8 -*-
"""
python API for content writers

USAGE
*****

Best way is to import all functions from this module:

from script_api import *

First thing to do is to set component:

set_component('httpd')

This is used when logging.

These functions are available:

* logging functions -- log_*
 * log message to stdout
* logging risk functions -- log_*_risk
 * log risk level -- so administrator know how risky is to inplace upgrade
* get_dest_dir -- get dir for storing configuration files
* set_component -- set component's name (for logging purposes)
* exit_* -- terminate execution with appropriate exit code
"""

from __future__ import unicode_literals, print_function
import os
import sys
import re
import shutil
import ConfigParser

from preup import utils, settings
from preup.utils import get_file_content, write_to_file

__all__ = (
    'log_debug',
    'log_info',
    'log_warning',
    'log_error',

    'log_extreme_risk',
    'log_high_risk',
    'log_medium_risk',
    'log_slight_risk',

    'get_dest_dir',
    'set_component',

    'exit_error',
    'exit_fail',
    'exit_failed',
    'exit_fixed',
    'exit_informational',
    'exit_not_applicable',
    'exit_informational',
    'exit_pass',
    'exit_unknown',
    'check_rpm_to',
    'check_applies_to',
    'solution_file',
    'switch_to_content',
    'service_is_enabled',
    'is_dist_native',
    'get_dist_native_list',

    'PREUPGRADE_CACHE',
    'VALUE_RPM_QA',
    'VALUE_ALLCHANGED',
    'VALUE_CONFIGCHANGED',
    'VALUE_PASSWD',
    'VALUE_CHKCONFIG',
    'VALUE_GROUP',
    'VALUE_RPMTRACKEDFILES',
    'VALUE_ALLMYFILES',
    'VALUE_EXECUTABLES',
    'VALUE_RPM_RHSIGNED',
    'VALUE_TMP_PREUPGRADE',
    'COMMON_DIR',
    'SOLUTION_FILE',
    'POSTUPGRADE_DIR',
    'KICKSTART_DIR',
    'KICKSTART_README',
    'KICKSTART_SCRIPTS',
    'KICKSTART_POSTUPGRADE',
    'MIGRATE',
    'UPGRADE',
    'HOME_DIRECTORY_FILE',
    'USER_CONFIG_FILE',
    'PREUPG_API_VERSION',
    'DEVEL_MODE',
    'DIST_NATIVE',
    'SPECIAL_PKG_LIST',
    'NOAUTO_POSTUPGRADE_D',
)

CACHE = "/var/cache/preupgrade"
PREUPGRADE_CACHE = os.path.join(CACHE, "common")
PREUPGRADE_CONFIG = settings.PREUPG_CONFIG_FILE
VALUE_RPM_QA = os.path.join(PREUPGRADE_CACHE, "rpm_qa.log")
VALUE_ALLCHANGED = os.path.join(PREUPGRADE_CACHE, "rpm_Va.log")
VALUE_CONFIGCHANGED = os.path.join(PREUPGRADE_CACHE, "rpm_etc_Va.log")
VALUE_PASSWD = os.path.join(PREUPGRADE_CACHE, "passwd.log")
VALUE_CHKCONFIG = os.path.join(PREUPGRADE_CACHE, "chkconfig.log")
VALUE_GROUP = os.path.join(PREUPGRADE_CACHE, "group.log")
VALUE_RPMTRACKEDFILES = os.path.join(PREUPGRADE_CACHE, "rpmtrackedfiles.log")
VALUE_ALLMYFILES = os.path.join(PREUPGRADE_CACHE, "allmyfiles.log")
VALUE_EXECUTABLES = os.path.join(PREUPGRADE_CACHE, "executable.log")
VALUE_RPM_RHSIGNED = os.path.join(PREUPGRADE_CACHE, "rpm_rhsigned.log")
VALUE_TMP_PREUPGRADE = os.environ['XCCDF_VALUE_TMP_PREUPGRADE']
SOLUTION_FILE = os.environ['XCCDF_VALUE_SOLUTION_FILE']
try:
    MIGRATE = os.environ['XCCDF_VALUE_MIGRATE']
    UPGRADE = os.environ['XCCDF_VALUE_UPGRADE']
except KeyError:
    MIGRATE = 1
    UPGRADE = 1
try:
    DEVEL_MODE = os.environ['XCCDF_VALUE_DEVEL_MODE']
except KeyError:
    DEVEL_MODE = 0

try:
    DIST_NATIVE = os.environ['XCCDF_VALUE_DIST_NATIVE']
except KeyError:
    DIST_NATIVE = 'sign'
POSTUPGRADE_DIR = os.path.join(VALUE_TMP_PREUPGRADE, "postupgrade.d")
KICKSTART_DIR = os.path.join(VALUE_TMP_PREUPGRADE, "kickstart")
KICKSTART_README = os.path.join(KICKSTART_DIR, "README")
KICKSTART_SCRIPTS = os.path.join(KICKSTART_DIR, "scripts")
KICKSTART_POSTUPGRADE = KICKSTART_SCRIPTS
COMMON_DIR = os.path.join(os.environ['XCCDF_VALUE_REPORT_DIR'], "common")
SPECIAL_PKG_LIST = os.path.join(KICKSTART_DIR, 'special_pkg_list')
NOAUTO_POSTUPGRADE_D = os.path.join(VALUE_TMP_PREUPGRADE, 'noauto_postupgrade.d')

HOME_DIRECTORY_FILE = ""
USER_CONFIG_FILE = 0

PREUPG_API_VERSION=1

component = "unknown"


################
# RISK LOGGING #
################

def log_risk(severity, message):
    """
    log risk level to stderr
    """
    print("INPLACERISK: %s: %s\n" % (severity, message.encode(settings.defenc)), end="", file=sys.stderr)


def log_extreme_risk(message):
    """
    log_extreme_risk(message)

    Inplace upgrade is impossible.
    """
    log_risk("EXTREME", message)


def log_high_risk(message):
    """
    log_high_risk(message)

    Administrator has to inspect and correct upgraded system so
    inplace upgrade can be used.
    """
    log_risk("HIGH", message)


def log_medium_risk(message):
    """
    log_medium_risk(message)

    inplace upgrade is possible; system after upgrade may be unstable
    """
    log_risk("MEDIUM", message)


def log_slight_risk(message):
    """
    log_slight_risk(message)

    no issues found; although there are some unexplored areas
    """
    log_risk("SLIGHT", message)


##################
# STDOUT LOGGING #
##################

def log(severity, message, component_arg=None):
    """log message to stdout"""
    global component
    comp_show = component_arg or component
    print("%s %s: %s\n" % (severity, comp_show, message.encode(settings.defenc)), end="", file=sys.stdout)


def log_error(message, component_arg=None):
    """
    log_error(message, component=None) -> None

    log message to stdout with severity error
    if you would like to change component temporary, you may pass it as argument

    use this severity if your script found something severe
    which may cause malfunction on new system
    """
    log('ERROR', message, component_arg)


def log_warning(message, component_arg=None):
    """
    log_warning(message, component_arg=None) -> None

    log message to stdout with severity warning
    if you would like to change component temporary, you may pass it as argument

    important finding, administrator of system should be aware of this
    """
    log('WARNING', message, component_arg)


def log_info(message, component_arg=None):
    """
    log_info(message, component_arg=None) -> None

    log message to stdout with severity info
    if you would like to change component temporary, you may pass it as argument

    informational message
    """
    log('INFO', message, component_arg)


def log_debug(message, component_arg=None):
    """
    log_debug(message, component_arg=None) -> None

    log message to stdout with severity debug
    if you would like to change component temporary, you may pass it as argument

    verbose information, may help with script debugging
    """
    log('DEBUG', message, component_arg)


#########
# UTILS #
#########

def get_dest_dir():
    """
    get_dest_dir()

    return absolute path to directory, where you should store files
    """
    return os.environ['XCCDF_VALUE_TMP_PREUPGRADE']


def shorten_envs():
    """make all the oscap's environemt variables shorter"""
    envs = os.environ
    prefixes = ('XCCDF_VALUE_', 'XCCDF_RESULT_')
    for env_key, env_value in envs.items():
        for prefix in prefixes:
            if env_key.startswith(prefix):
                os.environ[env_key.replace(prefix, '')] = env_value


def set_component(c):
    """configure name of component globally (it will be used in logging)"""
    global component
    component = c


# These are shortcut functions for:
#   sys.exit(int(os.environ['FAIL']))

def exit_fail():
    """
    The test failed.
    Moving to new release with this configuration will result in malfunction.
    """
    sys.exit(int(os.environ['XCCDF_RESULT_FAIL']))


def exit_failed():
    """
    The test failed.

    Moving to new release with this configuration will result in malfunction.
    """
    sys.exit(int(os.environ['XCCDF_RESULT_FAIL']))


def exit_error():
    """
    An error occurred and test could not complete.

    (script failed while doing its job)
    """
    sys.exit(int(os.environ['XCCDF_RESULT_ERROR']))


def exit_pass():
    """Test passed."""
    sys.exit(int(os.environ['XCCDF_RESULT_PASS']))


def exit_unknown():
    """Could not tell what happened."""
    sys.exit(int(os.environ['XCCDF_RESULT_UNKNOWN']))


def exit_not_applicable():
    """Rule did not apply to test target. (e.g. package is not installed)"""
    sys.exit(int(os.environ['XCCDF_RESULT_NOT_APPLICABLE']))


def exit_fixed():
    """Rule failed, but was later fixed."""
    sys.exit(int(os.environ['XCCDF_RESULT_FIXED']))


def exit_informational():
    """Rule failed, but was later fixed."""
    sys.exit(int(os.environ['XCCDF_RESULT_INFORMATIONAL']))


def switch_to_content():
    """Function for switch to the content directory"""
    os.chdir(os.environ['CURRENT_DIRECTORY'])


def is_pkg_installed(pkg_name):
    lines = get_file_content(VALUE_RPM_QA, "rb", True)
    found = [x for x in lines if x.split()[0] == pkg_name]
    if found:
        return True
    else:
        return False


def check_applies_to(check_applies=""):
    not_applicable = 0
    if check_applies != "":
        rpms = check_applies.split(',')
        for rpm in rpms:
            if not (is_pkg_installed(rpm) and is_dist_native(rpm)):
                log_info("Package %s is not installed or it is not signed by Red Hat." % rpm)
                not_applicable = 1
    if not_applicable:
        exit_not_applicable()


def check_rpm_to(check_rpm="", check_bin=""):
    not_applicable = 0

    if check_rpm != "":
        rpms = check_rpm.split(',')
        lines = get_file_content(VALUE_RPM_QA, "rb", True)
        for rpm in rpms:
            lst = filter(lambda x: rpm == x.split('\t')[0], lines)
            if not lst:
                log_info("Package %s is not installed." % rpm)
                not_applicable = 1

    if check_bin != "":
        binaries = check_bin.split(',')
        for binary in binaries:
            cmd = "which %s" % binary
            if utils.run_subprocess(cmd, print_output=False, shell=True) != 0:
                log_info("Binary %s is not installed." % binary)
                not_applicable = 1

    if not_applicable:
        log_high_risk("Please, install all required packages (and binaries) and run preupg again to process check properly.")
        exit_fail()


def solution_file(message):
    write_to_file(os.path.join(os.environ['CURRENT_DIRECTORY'], SOLUTION_FILE), "a+b", message)


def service_is_enabled(service_name):
    """Returns true if given service is enabled on any runlevel"""
    return_value = False
    lines = get_file_content(VALUE_CHKCONFIG, "rb", True)
    for line in lines:
        if re.match('^%s.*:on' % service_name, line):
            return_value = True
            break
    return return_value


def config_file_changed(config_file_name):
    """
    Searches cached data in VALUE_CONFIGCHANGED

    returns:
    True if given config file has been changed
    False if given config file hasn't been changed
    """
    config_changed = False
    try:
        lines = get_file_content(VALUE_CONFIGCHANGED, "rb", True)
        for line in lines:
            if line.find(config_file_name) != -1:
                config_changed = True
                break
    except:
        pass
    return config_changed


def backup_config_file(config_file_name):
    """Copies specified file into VALUE_TMP_PREUPGRADE, keeping file structure"""
    try:
        # report error if file doesn't exist
        if not os.path.isfile(config_file_name):
            return 1

        # don't do anything if config file was not changed
        if not config_file_changed(config_file_name):
            return 2

        # stripping / from beginning is necessary to concat paths properly
        os.mkdir(os.path.join(VALUE_TMP_PREUPGRADE, os.path.dirname(config_file_name.strip("/"))))
    except OSError:
        # path probably exists, it's ok
        pass
    shutil.copyfile(config_file_name, os.path.join(VALUE_TMP_PREUPGRADE, config_file_name.strip("/")))


def is_dist_native(pkg):
    """
    is_dist_native function return only True or False
    return False if package is not installed and of course information log.
    Case DEVEL_MODE is turn off then return True if package is signed or False if not.
    Case DEVEL_MODE is turn on:
    DIST_NATIVE = sign: return True if is RH_SIGNED else return False
    DIST_NATIVE = all: always return True
    DIST_NATIVE = path_to_file: return True if package is in file else return False
    """

    rpm_qa = get_file_content(VALUE_RPM_QA, "rb", True)
    found = [x for x in rpm_qa if x.split()[0] == pkg]
    if not found:
        log_warning("Package %s is not installed on Red Hat Enterprise Linux system.")
        return False

    rpm_signed = get_file_content(VALUE_RPM_RHSIGNED, "rb", True)
    found = [x for x in rpm_signed if x.split()[0] == pkg]
    if int(DEVEL_MODE) == 0:
        if found:
            return True
        else:
            return False
    else:
        if DIST_NATIVE == "all":
            return True
        if DIST_NATIVE == "sign":
            if found:
                return True
            else:
                return False
        if os.path.exists(DIST_NATIVE):
            list_native = get_file_content(DIST_NATIVE)
            if pkg in list_native:
                return True
        return False


def get_dist_native_list():
    """
    returns list of all installed native packages
    """

    native_pkgs = []
    tmp = get_file_content(VALUE_RPM_QA, "rb", True)
    pkgs = [i.split("\t")[0] for i in tmp]
    for pkg in pkgs:
        if(is_dist_native(pkg) is True):
            native_pkgs.append(pkg)
    return native_pkgs


def load_pa_configuration():
    """ Loads preupgrade-assistant configuration file """
    global HOME_DIRECTORY_FILE
    global USER_CONFIG_FILE
    global RH_SIGNED_PKGS

    if not os.path.exists(PREUPGRADE_CONFIG):
        log_error("Configuration file $PREUPGRADE_CONFIG is missing or is not readable!")
        exit_error()

    config = ConfigParser.RawConfigParser(allow_no_value=True)
    config.read(PREUPGRADE_CONFIG)
    section = 'preupgrade-assistant'
    home_option = 'home_directory_file'
    user_file = 'user_config_file'
    if config.has_section(section):
        if config.has_option(section, home_option):
            HOME_DIRECTORY_FILE = config.get(section, home_option)
        if config.has_option(section, user_file):
            USER_CONFIG_FILE = config.get(section, user_file)


def print_home_dirs(user_name=""):
    """ Loads preupgrade-assistant configuration file """
    if not os.path.exists(PREUPGRADE_CONFIG):
        log_error("Configuration file $PREUPGRADE_CONFIG is missing or is not readable!")
        exit_error()

    config = ConfigParser.RawConfigParser(allow_no_value=True)
    home_option = 'home-dirs'
    try:
        if USER_CONFIG_FILE == 'enabled' and user_name == "":
            config.read(PREUPGRADE_CONFIG)
            return config.options(home_option)
        user_home_dir = os.path.join('/home', user_name, HOME_DIRECTORY_FILE)
        if not os.path.exists(user_home_dir):
            return 0
        config.read(user_home_dir)
        return config.options(home_option)
    except ConfigParser.NoSectionError:
        pass
    except ConfigParser.NoOptionError:
        pass


def add_pkg_to_kickstart(pkg_name):
    empty = False
    if isinstance(pkg_name, list):
        # list of packages
        if len(pkg_name) == 0:
            empty = True
    else:
        # string - pkg_name delimited by whitespace
        if len(pkg_name.strip()) == 0:
            empty = True
        else:
            # make list from string
            pkg_name = pkg_name.strip().split()
    if empty is True:
        log_debug("Missing parameters! Any package will be added.")
        return 1
    for pkg in pkg_name.split():
        write_to_file(SPECIAL_PKG_LIST, "a+b", pkg)
    return 0


load_pa_configuration()

shorten_envs()



from __future__ import unicode_literals
import unittest
import tempfile
import shutil
import os

from preup.application import Application
from preup.conf import Conf, DummyConf
from preup.cli import CLI
from preup import settings, remediate, utils, xml_manager
from preup.report_parser import ReportParser

import base

class TestPreupg(base.TestCase):
    def setUp(self):
        self.temp_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.temp_dir)

    def test_all(self):

        """Basic test for whole program"""

        conf = {
            "contents": "tests/FOOBAR6_7/dummy_preupg/all-xccdf.xml",
            "profile": "xccdf_preupg_profile_default",
            "result_dir": self.temp_dir,
            "skip_common": True,
            "temp_dir": self.temp_dir,
            "id": None,
            "debug": True,  # so root check won't fail
        }

        dc = DummyConf(**conf)
        cli = CLI(["--contents", "tests/FOOBAR6_7/dummy_preupg/all-xccdf.xml"])
        a = Application(Conf(dc, settings, cli))
        # Prepare all variables for test
        a.conf.source_dir = os.getcwd()
        a.content = a.conf.contents
        a.basename = os.path.basename(a.content)
        self.assertEqual(a.run_scan(), 0)

    def test_migrate(self):
        """Basic test for whole program"""

        conf = {
            "contents": "tests/FOOBAR6_7/dummy_preupg/all-xccdf.xml",
            "profile": "xccdf_preupg_profile_default",
            "result_dir": self.temp_dir,
            "skip_common": True,
            "temp_dir": self.temp_dir,
            "id": None,
            "debug": True,  # so root check won't fail
            "mode": 'migrate'
        }

        dc = DummyConf(**conf)
        cli = CLI(["--contents", "tests/FOOBAR6_7/dummy_preupg/all-xccdf.xml", "--mode", "migrate"])
        a = Application(Conf(dc, settings, cli))
        # Prepare all variables for test
        a.conf.source_dir = os.getcwd()
        a.content = a.conf.contents
        a.basename = os.path.basename(a.content)
        self.assertEqual(a.run_scan(), 0)
        rp = ReportParser(os.path.join(self.temp_dir, "result.xml"))
        rp.modify_result_path(self.temp_dir, "FOOBAR6_7", 'migrate')
        found_migrate = 0
        found_upgrade = 0
        for values in rp.get_nodes(rp.target_tree, "Value", ".//"):
            if values.get("id").endswith("_preupg_state_migrate"):
                for value in rp.get_nodes(values, "value"):
                    if int(value.text) == 1:
                        found_migrate = 1
            if values.get("id").endswith("_preupg_state_upgrade"):
                for value in rp.get_nodes(values, "value"):
                    if int(value.text) == 0:
                        found_upgrade = 1
        self.assertEquals(found_migrate, 1)
        self.assertEquals(found_upgrade, 1)

    def test_upgrade(self):
        """Basic test for whole program"""

        conf = {
            "contents": "tests/FOOBAR6_7/dummy_preupg/all-xccdf.xml",
            "profile": "xccdf_preupg_profile_default",
            "result_dir": self.temp_dir,
            "skip_common": True,
            "temp_dir": self.temp_dir,
            "id": None,
            "debug": True,  # so root check won't fail
            "mode": 'upgrade'
        }

        dc = DummyConf(**conf)
        cli = CLI(["--contents", "tests/FOOBAR6_7/dummy_preupg/all-xccdf.xml", "--mode", "upgrade"])
        a = Application(Conf(dc, settings, cli))
        # Prepare all variables for test
        a.conf.source_dir = os.getcwd()
        a.content = a.conf.contents
        a.basename = os.path.basename(a.content)
        self.assertEqual(a.run_scan(), 0)
        rp = ReportParser(os.path.join(self.temp_dir, "result.xml"))
        rp.modify_result_path(self.temp_dir, "FOOBAR6_7", 'upgrade')
        found_migrate = 0
        found_upgrade = 0
        for values in rp.get_nodes(rp.target_tree, "Value", ".//"):
            if values.get("id").endswith("_preupg_state_migrate"):
                for value in rp.get_nodes(values, "value"):
                    if int(value.text) == 0:
                        found_migrate = 1
            if values.get("id").endswith("_preupg_state_upgrade"):
                for value in rp.get_nodes(values, "value"):
                    if int(value.text) == 1:
                        found_upgrade = 1
        self.assertEquals(found_migrate, 1)
        self.assertEquals(found_upgrade, 1)


class TestXMLUpdates(base.TestCase):
    def setUp(self):
        self.content = "tests/FOOBAR6_7/dummy_preupg/all-xccdf.xml"
        self.test_content = self.content+".test"

    def tearDown(self):
        os.remove(self.test_content)

    def test_platform_tag(self):
        shutil.copyfile(self.content, self.test_content)
        rp = ReportParser(self.test_content)
        rp.modify_platform_tag("12")

        found = 0
        for platform in rp.get_nodes(rp.target_tree, "platform"):
            if "cpe:/o:redhat:enterprise_linux:12" in platform.get('idref'):
                found = 1
        self.assertTrue(found)

    def test_result_dirs_tmp_preupgrade(self):
        shutil.copyfile(self.content, self.test_content)
        rp = ReportParser(self.test_content)
        result_path = "/abc/def"
        rp.modify_result_path(result_path, "FOOBAR6_7", 'migrate')
        found_tmp = 0

        for values in rp.get_nodes(rp.target_tree, "Value", ".//"):
            if values.get("id").endswith("_preupg_state_tmp_preupgrade"):
                for value in rp.get_nodes(values, "value"):
                    if value.text == result_path:
                        found_tmp = 1

        self.assertEquals(found_tmp, 1)

    def test_result_dirs_current_dir(self):
        shutil.copyfile(self.content, self.test_content)
        rp = ReportParser(self.test_content)
        result_path = "/abc/efg"
        scenario = 'FOOBAR6_7'
        rp.modify_result_path(result_path, scenario, 'migrate')
        found_current = 0
        for values in rp.get_nodes(rp.target_tree, "Value", ".//"):
            if values.get("id").endswith("_preupg_state_current_directory"):
                for value in rp.get_nodes(values, "value"):
                    result_dir = result_path+"/"+scenario+"/dummy_preupg"
                    if value.text == result_dir:
                        found_current = 1

        self.assertEquals(found_current, 1)


class TestCLI(base.TestCase):
    def test_opts(self):
        """ basic test of several options """
        conf = {
            "scan": "FOOBAR6_7",
            "temp_dir": 'd',
            "skip_common": False,
            "contents": "content/FOOBAR6_7",
            "id": 1,
            "list": True,
            "verbose": 1,
            "text": True,
            "cleanup": True,
            "mode": 'upgrade',
        }
        dc = DummyConf(**conf)
        cli = CLI(["--scan", "FOOBAR6_7", "--skip-common", "--list",
                   "--verbose", "--text",
                   "--contents", "content/FOOBAR6_7", "--cleanup", "--mode", "upgrade"])
        a = Application(Conf(cli.opts, dc, cli))

        self.assertTrue(a.conf.skip_common)
        self.assertEqual(a.conf.contents, "content/FOOBAR6_7")
        self.assertTrue(a.conf.list)
        self.assertTrue(a.conf.text)
        self.assertTrue(a.conf.cleanup)
        self.assertEqual(int(a.conf.verbose), 1)
        self.assertEqual(a.conf.temp_dir, "d")
        self.assertEqual(a.conf.scan, "FOOBAR6_7")
        self.assertEqual(a.conf.mode, "upgrade")


class TestHashes(base.TestCase):
    def setUp(self):
        self.dir_name = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.dir_name)

    def test_hashes(self):
        text_to_hash="""
            This is preupgrade assistant test has string"
        """
        self.dir_name = "tests/hashes"
        os.mkdir(self.dir_name)
        utils.write_to_file(os.path.join(self.dir_name, "post_script"), 'wb', text_to_hash)
        remediate.hash_postupgrade_file(False, self.dir_name)
        return_value = remediate.hash_postupgrade_file(False, self.dir_name, check=True)
        self.assertTrue(return_value)


class TestSolutionReplacement(base.TestCase):
    def setUp(self):
        self.extension = "html"

    def test_solution_bold_tag(self):
        solution_text = ['This is solution text [bold: provided as text ] by check script']
        expected_text = ['This is solution text <b> provided as text </b> by check script']
        line = xml_manager.tag_formating(solution_text, self.extension)
        self.assertEqual(expected_text, line)

    def test_solution_www_link_tag(self):
        solution_text = ['This is solution text [link: http://127.0.0.1/all-xccdf.html ] by check script']
        expected_text = ['This is solution text <a href="http://127.0.0.1/all-xccdf.html">http://127.0.0.1/all-xccdf.html</a> by check script']
        line = xml_manager.tag_formating(solution_text, self.extension)
        self.assertEqual(expected_text, line)

    def test_solution_file_link_tag(self):
        solution_text = ['This is solution text [link: description.txt ] by check script']
        expected_text = ['This is solution text <a href="./description.txt">description.txt</a> by check script']
        line = xml_manager.tag_formating(list(solution_text), self.extension)
        self.assertEqual(list(expected_text), line)

    def test_solution_wrong_link_tag(self):
        solution_text = ['This is solution text [link: /var/cache/description.txt] by check script']
        expected_text = ['This is solution text [link: /var/cache/description.txt] by check script']
        line = xml_manager.tag_formating(solution_text, self.extension)
        self.assertEqual(expected_text, line)

    def test_solution_file_bold_tag(self):
        solution_text = ['This is solution text [link: description.txt ] by [bold: check script ]']
        expected_text = ['This is solution text <a href="./description.txt">description.txt</a> by <b> check script </b>']
        line = xml_manager.tag_formating(solution_text, self.extension)
        self.assertEqual(expected_text, line)

    def test_solution_www_bold_tag(self):
        solution_text = ['This is solution text [link: http://127.0.0.1/description.txt ] by [bold: check script ]']
        expected_text = ['This is solution text <a href="http://127.0.0.1/description.txt">http://127.0.0.1/description.txt</a> by <b> check script </b>']
        line = xml_manager.tag_formating(solution_text, self.extension)
        self.assertEqual(expected_text, line)


class TestScenario(base.TestCase):
    def setUp(self):
        self.temp_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.temp_dir)

    def test_correct_content_scenario(self):
        """
        Basic test for whole program
        """
        conf = {
            "contents": "tests/FOOBAR6_77/dummy_preupg/all-xccdf.xml",
            "profile": "xccdf_preupg_profile_default",
            "result_dir": self.temp_dir,
            "skip_common": True,
            "temp_dir": self.temp_dir,
            "id": None,
            "debug": True,  # so root check won't fail
        }

        dc = DummyConf(**conf)
        cli = CLI(["--contents", "tests/FOOBAR6_77/dummy_preupg/all-xccdf.xml"])
        a = Application(Conf(dc, settings, cli))
        # Prepare all variables for test
        a.conf.source_dir = os.getcwd()
        a.content = a.conf.contents
        a.basename = os.path.basename(a.content)
        self.assertEqual(a.get_scenario(), "FOOBAR6_77")

    def test_migration_content_scenario(self):
        """
        Basic test for whole program
        """
        conf = {
            "contents": "tests/FOOBAR6_CENTOS6/dummy_preupg/all-xccdf.xml",
            "profile": "xccdf_preupg_profile_default",
            "result_dir": self.temp_dir,
            "skip_common": True,
            "temp_dir": self.temp_dir,
            "id": None,
            "debug": True,  # so root check won't fail
        }

        dc = DummyConf(**conf)
        cli = CLI(["--contents", "tests/FOOBAR6_CENTOS6/dummy_preupg/all-xccdf.xml"])
        a = Application(Conf(dc, settings, cli))
        # Prepare all variables for test
        a.conf.source_dir = os.getcwd()
        a.content = a.conf.contents
        a.basename = os.path.basename(a.content)
        self.assertEqual(a.get_scenario(), "FOOBAR6_CENTOS6")

    def test_wrong_content_scenario(self):
        """
        Basic test for whole program
        """
        conf = {
            "contents": "tests/FOOBAR6_7A/dummy_preupg/all-xccdf.xml",
            "profile": "xccdf_preupg_profile_default",
            "result_dir": self.temp_dir,
            "skip_common": True,
            "temp_dir": self.temp_dir,
            "id": None,
            "debug": True,  # so root check won't fail
        }

        dc = DummyConf(**conf)
        cli = CLI(["--contents", "tests/FOOBAR6_7A/dummy_preupg/all-xccdf.xml"])
        a = Application(Conf(dc, settings, cli))
        # Prepare all variables for test
        a.conf.source_dir = os.getcwd()
        a.content = a.conf.contents
        a.basename = os.path.basename(a.content)
        self.assertEqual(a.get_scenario(), None)


class TestPreupgradePrefix(base.TestCase):
    def setUp(self):
        settings.prefix = 'preupgrade'

    def test_correct_prefix(self):
        version = utils.get_assessment_version('FOOBAR6_7')
        self.assertEqual(version, ['6', '7'])

    def test_wrong_prefix(self):
        version = utils.get_assessment_version('FOOBAR6_CENTOS6')
        self.assertEqual(version, None)


class TestPremigratePrefix(base.TestCase):
    def setUp(self):
        settings.prefix = 'premigrate'

    def test_correct_prefix(self):
        version = utils.get_assessment_version('FOOBAR6_CENTOS6')
        self.assertEqual(version, ['6', '6'])

    def test_wrong_prefix(self):
        version = utils.get_assessment_version('FOOBAR6_7')
        self.assertEqual(version, None)


def suite():
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTest(loader.loadTestsFromTestCase(TestPreupg))
    suite.addTest(loader.loadTestsFromTestCase(TestCLI))
    suite.addTest(loader.loadTestsFromTestCase(TestHashes))
    suite.addTest(loader.loadTestsFromTestCase(TestSolutionReplacement))
    suite.addTest(loader.loadTestsFromTestCase(TestXMLUpdates))
    suite.addTest(loader.loadTestsFromTestCase(TestScenario))
    suite.addTest(loader.loadTestsFromTestCase(TestPreupgradePrefix))
    suite.addTest(loader.loadTestsFromTestCase(TestPremigratePrefix))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=3).run(suite())




from __future__ import unicode_literals, print_function
import unittest
import tempfile
import os
import shutil

import base

from preup_creator.ui_helper import UIHelper
from preup import utils


def load_file(filename):
    try:
        lines = utils.get_file_content(filename, "rb", True)
        lines = [x.strip() for x in lines]
    except IOError:
        assert False
    return lines


class TestCreator(base.TestCase):

    tempdir = ""
    upgrade_dir = ""
    puh = None
    group_name = "foobar_group"
    content_name = "foobar_content"

    def setUp(self):
        self.tempdir = tempfile.mkdtemp()
        self.upgrade_dir = os.path.join(self.tempdir, "FOOBAR6_7")
        os.makedirs(self.upgrade_dir)
        self.puh = UIHelper(self.upgrade_dir)
        self.puh._group_name = self.group_name
        self.puh._content_name = self.content_name
        self.puh.check_script = True
        self.puh.solution_file = True
        content_dict = {}
        content_dict['check_script'] = "foobar_check_script.sh"
        content_dict['solution'] = "foobar_solution.txt"
        content_dict['content_title'] = "foobar_test_title"
        content_dict['content_description'] = "Foobar content test description"
        self.puh.content_dict = content_dict
        self.puh.content_path = os.path.join(self.puh.get_group_name(), self.puh.get_content_name())
        if os.path.exists(self.puh.get_content_path()):
            shutil.rmtree(self.puh.get_content_path())
        os.makedirs(self.puh.get_content_path())
        self.puh.prepare_content_env()
        self.puh.create_final_content()

    def tearDown(self):
        shutil.rmtree(self.tempdir)

    def test_content_ini(self):
        content_ini = os.path.join(self.puh.get_content_path(), self.puh.get_content_ini_file())
        self.assertEqual(content_ini, os.path.join(self.upgrade_dir,
                                                   self.group_name,
                                                   self.content_name,
                                                   self.content_name + '.ini'))
        lines = load_file(content_ini)
        expected_ini = ['[preupgrade]',
                        'check_script = foobar_check_script.sh',
                        'content_description = Foobar content test description',
                        'solution = foobar_solution.txt',
                        'content_title = foobar_test_title',
                        '']

        self.assertListEqual(lines, expected_ini)

    def test_group_ini(self):
        group_ini = os.path.join(self.puh.get_upgrade_path(),
                                 self.puh.get_group_name(), 'group.ini')
        self.assertEqual(group_ini, os.path.join(self.upgrade_dir,
                                                 self.group_name,
                                                 'group.ini'))
        lines = load_file(group_ini)
        expected_group_ini = ['[preupgrade]',
                              'group_title = Title for foobar_group',
                              '']
        self.assertListEqual(lines, expected_group_ini)

    def test_check_script(self):
        check_script = os.path.join(self.puh.get_content_path(), self.puh.get_check_script())
        lines = load_file(check_script)
        exp_script = ['#!/bin/bash',
                      '',
                      '. /usr/share/preupgrade/common.sh',
                      '',
                      '#END GENERATED SECTION',
                      '',
                      "### For more information see 'man preupg-content-creator' or 'man preupgrade-assistant-api'.",
                      '']
        self.assertListEqual(lines, exp_script)


def suite():
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTest(loader.loadTestsFromTestCase(TestCreator))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=3).run(suite())



import unittest
from tests import test_preup
from tests import test_xml
from tests import test_oscap
from tests import test_generation
from tests import test_api
from tests import test_kickstart


def suite():
    suite = unittest.TestSuite()
    suite.addTests(test_preup.suite())
    suite.addTests(test_xml.suite())
    suite.addTests(test_oscap.suite())
    suite.addTests(test_api.suite())
    suite.addTests(test_kickstart.suite())
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=3).run(suite())



"""
The class is used for common stuff issues like
generating common logs, coping these common logs
to assessment
"""

from __future__ import unicode_literals
import os
import platform
import datetime
import shutil
from distutils import dir_util
from preup import utils
from preup.logger import log_message
from preup import settings


def get_add_on_name(filename, add_on):
    """Function returns the server name with add_on"""
    return filename + "-" + add_on


class Common(object):

    """Class handles with common log files"""

    def __init__(self, conf):
        self.conf = conf
        self.cwd = ""
        self.lines = utils.get_file_content(self.conf.common_script, "rb", method=True)
        self.common_result_dir = ""

    def common_logfiles(self, filename):
        """build path for provided filename"""
        return os.path.join(self.get_common_dir(), filename)

    def get_common_dir(self):
        """Function returns common dir"""
        return os.path.join(self.conf.cache_dir, settings.common_name)

    def switch_dir(self):
        """Switch to current directory"""
        com_dir = self.get_common_dir()
        if not os.path.exists(com_dir):
            utils.check_or_create_temp_dir(com_dir)
        self.cwd = os.getcwd()
        os.chdir(self.get_common_dir())

    def switch_back_dir(self):
        """Function switch back to self.cwd"""
        os.chdir(self.cwd)

    def common_results(self):
        """run common scripts"""
        log_message("Gathering logs used by preupgrade assistant:")
        self.switch_dir()
        try:
            max_length = max(max([len(x.split("=", 4)[3]) for x in self.lines]), len(settings.assessment_text))
            # Log files which will not be updated
            # when RPM database is not changed
            for counter, line in enumerate(self.lines):
                line = line.strip()
                if line.startswith("#"):
                    continue
                cmd, log_file, dummy_bash_value, name, values = line.split("=", 4)
                log_message("%s : %.2d/%d ...running" % (name.ljust(max_length),
                                                         counter+1,
                                                         len(self.lines)),
                            new_line=False, log=False)
                start_time = datetime.datetime.now()
                common_file_path = self.common_logfiles(log_file)
                utils.run_subprocess(cmd, output=common_file_path, shell=True)
                end_time = datetime.datetime.now()
                diff = end_time - start_time
                log_message(" %sfinished (time %.2d:%.2ds)" % ('\b' * 8,
                                                               diff.seconds / 60,
                                                               diff.seconds % 60),
                            log=False)
                # os.chmod(common_file_path, 0640)
            self.switch_back_dir()
        except IOError:
            return 0
        else:
            return 1

    def copy_common_files(self):
        """run common scripts"""
        self.switch_dir()

        try:
            for line in self.lines:
                if line.strip().startswith("#"):
                    continue
                values = line.strip().split("=", 5)
                if values[4] == "YES":
                    shutil.copyfile(values[1],
                                    os.path.join(self.conf.result_dir,
                                                 "kickstart",
                                                 values[5]))
                else:
                    if os.path.exists(os.path.join(self.conf.result_dir,
                                                   values[1])):
                        os.remove(values[1])
        except IOError:
            return 0
        else:
            self.switch_back_dir()
            return 1

    def get_default_name(self, filename):
        """Function returns a full default name need for symlink"""
        return os.path.join(self.common_result_dir, filename)

    def remove_common_symlink(self, filename):
        """Function removes a symlink if it already exists"""
        filename_remove = self.get_default_name(filename)
        if os.path.islink(filename_remove):
            os.unlink(filename_remove)

    def create_common_symlink(self, filename, variant):
        """
        Function removes previous link if exists
        and then creates a new one
        """
        self.remove_common_symlink(filename)
        sym_link_name = filename.replace(variant, 'default')
        os.symlink(os.path.join(self.common_result_dir,
                                platform.machine(),
                                filename),
                   os.path.join(self.common_result_dir, sym_link_name))

    def copy_kickstart_files(self, dir_name, variant):
        """
        Function copies files which are needed by kickstart

        :param source dir_name:
        :return:
        """
        for file_name in settings.KS_FILES:
            target_file = os.path.join(settings.KS_DIR, file_name)
            orig_name = file_name.replace('default', variant)
            source_name = os.path.join(dir_name, platform.machine(), orig_name)
            if not os.path.exists(target_file) and os.path.exists(source_name):
                shutil.copyfile(source_name, target_file)

    def prep_symlinks(self, assessment_dir, scenario=""):
        """Prepare a symlinks for relevant architecture and Server Variant"""
        server_variant = utils.get_variant()
        if server_variant is None:
            return
        self.common_result_dir = os.path.join(assessment_dir, settings.common_name)
        # We need to copy /usr/share/preupgrade/RHEL6_7/common also in case of
        # usage --contents option. Some contents needs a /root/preupgrade/RHEL6_7/common
        # directory
        if self.conf.contents:
            usr_common_name = os.path.join(settings.source_dir, scenario, settings.common_name)
            if os.path.exists(usr_common_name):
                dir_util.copy_tree(usr_common_name, os.path.join(assessment_dir, settings.common_name))
        # We have repositories for i386 architecture but packages are built
        # sometimes as i686 architecture. That's problematic in some cases
        # so we solve this for now by this little hack ugly.
        if (not os.path.exists(os.path.join(self.common_result_dir, 'i686'))
           and os.path.exists(os.path.join(self.common_result_dir, 'i386'))):
            os.symlink(os.path.join(self.common_result_dir, 'i386'),
                       os.path.join(self.common_result_dir, 'i686'))
        add_ons = utils.get_addon_variant()
        dir_name = os.path.join(self.common_result_dir,
                                platform.machine())
        if not os.path.exists(dir_name):
            return
        server_variant_files = [files for files in os.listdir(dir_name) if files.startswith(server_variant) or files.startswith("Common")]
        self.copy_kickstart_files(self.common_result_dir, server_variant)
        for files in server_variant_files:
            # First create a default links to "ServerVariant_"
            if files.startswith(server_variant):
                self.create_common_symlink(files, server_variant)
            elif files.startswith("Common"):
                self.create_common_symlink(files, "Common")



import optparse
from optparse import OptionValueError

from preup.constants import *


class CLIKickstart(object):
    """ Class for processing data from commandline """

    def __init__(self, args=None):
        """ parse arguments """
        self.parser = optparse.OptionParser(usage=USAGE, description=PROGRAM_DESCRIPTION)

        #self.parser.usage = "%%prog [-v] <content_file>"

        self.add_args()
        if args:
            self.opts, self.args = self.parser.parse_args(args=args)
        else:
            self.opts, self.args = self.parser.parse_args()

    def add_args(self):
        self.parser.add_option(
            "--channels",
            metavar="CHANNELS",
            type=str,
            help="List of channels for RHEL packages. Base is add automatically."
        )


if __name__ == '__main__':
    x = CLIKickstart()
    print (x.args.id)





from __future__ import unicode_literals, print_function
import logging
import os
import sys
from preup import settings


# Currently we have only one log
logger = logging.getLogger('preupgrade-assistant')
try:
    hdlr = logging.FileHandler(os.path.join(settings.log_dir, "preupg.log"))
except IOError:
    # python 2.6 uses argument 'strm', where python 2.7 uses 'stream'
    # therefore don't rather specify the argument
    hdlr = logging.StreamHandler(sys.stdout)
formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
hdlr.setFormatter(formatter)
logger.addHandler(hdlr)

def set_level(level):
    logger.setLevel(level)

if(sys.version_info[0] == 2):
    def log_message(message, print_output=1, new_line=True, level=logging.INFO, log=True):
        """ if verbose, log `msg % args` to stdout """
        if int(print_output) > 0:
            sys.stdout.write(message.encode(settings.defenc))
            sys.stdout.flush()
            # This is used in case that we do not want to print the new line
            if new_line:
                sys.stdout.write("\n")
                sys.stdout.flush()
        if log:
            logger.log(level, message)
else:
    def log_message(message, print_output=1, new_line=True, level=logging.INFO, log=True):
        """ if verbose, log `msg % args` to stdout """
        if int(print_output) > 0:
            endline = "\n" if new_line else ""
            print(message, end=endline, file=sys.stdout, flush=True)
        if log:
            logger.log(level, message)



# -*- coding: utf-8 -*-

"""
Valid settings are:

# dir where results of analysis are stored
result_dir = "/var/tmp/preupgrade"

# xccdf profile
profile = "xccdf_preupg_profile_default"

# URL of frontend
frontend = ""

# name of dir with common files
common_name = "common"

# absolute path to dir with common files
common_dir = os.path.join(os.path.dirname(__file__),"..", common_name)

# path to file with definitions of common scripts
common_script = os.path.join(common_dir, "scripts.txt")

# name of file for results
results = 'results'

# print more data into stdout
verbose = 1

# skip generation of common files
skip_common = False

# SCE XML file
sce_xml = 'sce.xml'

# enable debugging
debug = False

# id of run in frontend
id = 123456

# prefix of tag in fccdf files
xccdf_tag = "xccdf_preupg_rule_"
"""

from __future__ import unicode_literals

class DummyConf(object):
    """
    Dummy conf class for Conf

    use it like this:
    conf = Conf(DummyConf(id=123, skip_common=True))
    """
    def __init__(self, **kwargs):
        self.settings = kwargs

    def __getattr__(self, name):
        try:
            return self.settings.get(name)
        except AttributeError:
            return object.__getattribute__(self, name)


class Conf(object):
    """
    configuration of preupgrade assistant

    merged values from CLI and settings.py
    """

    def __init__(self, *args):
        """
        *args - list of objects with settings attached as
                attributes to these objects
        priority:
            args[0] > args[1] > ...
        """
        self.settings = list(args)

    def insert(self, pos, settings):
        self.settings.insert(pos, settings)

    def __getattr__(self, name):
        for arg in self.settings:
            try:
                value = getattr(arg, name)
            except AttributeError:
                continue
            if value is not None and value != "":
                return value
            else:
                continue
        #return object.__getattribute__(self, name)
        return None



# -*- coding: utf-8 -*-

"""
This module serves for handling user interactions
"""

import os
import ConfigParser
import shutil
import sys

from distutils.util import strtobool
from preup import utils
from preup.utils import get_valid_scenario
from preup_creator import settings

from preup.settings import content_file as ALL_XCCDF_XML
section = 'preupgrade'


def get_user_input(message, any_input=False):
    """
    Function for command line messages

    :param message: prompt string
    :param default_yes: If the default value is YES
    :param any_input: if True, return input without checking it first
    :return: True or False, based on user's input
    """
    choice = '([y]/n)'

    if any_input:
        msg = '{0} '.format(message)
    else:
        msg = '{0} {1}? '.format(message, choice)

    while True:
        if(sys.version_info[0] == 2):
            user_input = raw_input(msg)
        else:
            user_input = input(msg)

        if not any_input:
            try:
                user_input = strtobool(user_input)
            except ValueError:
                print ('You have to type [y]es or [n]o.')
                continue

        return user_input


class UIHelper(object):

    def __init__(self, upgrade_path=None):
        self.content_dict = {}
        self.upgrade_path = upgrade_path
        self._group_name = ""
        self._content_name = ""
        self.content_path = None
        self.content_ini = ""
        self.check_script = True
        self.solution_file = True
        self.refresh_content = False

    def _init_dict(self):
        self.content_dict['content_description'] = ''

    @staticmethod
    def check_path(path, msg):
        if os.path.exists(path):
            choice = get_user_input(msg)
            if not choice:
                return None
        return True

    def get_group_name(self):
        return self._group_name

    def get_content_name(self):
        return self._content_name

    def _get_dict_value(self, name):
        try:
            return self.content_dict[name]
        except KeyError:
            return None

    def get_check_script(self):
        return self._get_dict_value('check_script')

    def get_solution_file(self):
        return self._get_dict_value('solution')

    def get_content_ini_file(self):
        return self.content_ini

    def get_content_path(self):
        return os.path.join(self.get_upgrade_path(), self.content_path)

    def get_upgrade_path(self):
        if self.upgrade_path.startswith("/"):
            return self.upgrade_path
        return os.path.join(os.getcwd(), self.upgrade_path)

    def specify_upgrade_path(self):
        if self.upgrade_path is None:
            self.upgrade_path = get_user_input(settings.upgrade_path, any_input=True)

        if not get_valid_scenario(self.upgrade_path):
            print ("Scenario '%s' is not valid.\nIt has to be like RHEL6_7 or CentOS7_RHEL7." % self.content_path)
            return None

        message = 'Path %s already exists.\nDo you want to create a content there?' % self.upgrade_path
        if UIHelper.check_path(self.get_upgrade_path(), message) is None:
            return None

        return True

    def prepare_content_env(self):
        self.content_path = os.path.join(self.get_group_name(), self.get_content_name())

    def get_content_info(self):
        self._group_name = get_user_input(settings.group_name, any_input=True)
        self._content_name = get_user_input(settings.content_name, any_input=True)
        self.prepare_content_env()
        if os.path.exists(self.get_content_path()):
            message = "Content %s already exists.\nDo you want to overwrite them?" % os.path.join(self.upgrade_path,
                                                                                                  self.content_path)
            if UIHelper.check_path(os.path.join(self.upgrade_path, self.content_path), message):
                # User would like to overwrite the content. We will delete them and and make newerone.
                self.refresh_content = True
        else:
            os.makedirs(self.get_content_path())
        checkscript = get_user_input(settings.check_script, any_input=True)
        if checkscript == "":
            checkscript = "check.sh"
        if UIHelper.check_path(os.path.join(self.get_content_path(), checkscript),
                               settings.check_path % checkscript) is None:
            self.check_script = False
        self.content_dict['check_script'] = checkscript
        solution = get_user_input(settings.solution_text, any_input=True)
        if solution == "":
            solution = "solution.txt"
        if UIHelper.check_path(os.path.join(self.get_content_path(), solution),
                               settings.check_path % solution) is None:
            self.solution_file = False
        self.content_dict['solution'] = solution
        self.content_dict['content_title'] = get_user_input(settings.content_title, any_input=True)
        response = get_user_input(settings.content_desc)
        if not response:
            self.content_dict['content_description'] = None
        else:
            desc = get_user_input(settings.content_desc_text, any_input=True)
            self.content_dict['content_description'] = desc

    def _create_ini_file(self):
        """
        INI file should look like
        [preupgrade]
        content_title: Bacula Backup Software
        author: Petr Hracek <phracek@redhat.com>
        content_description: This module verifies the directory permissions for the Bacula service.
        solution: bacula.txt
        check_script: check.sh
        applies_to: bacula-common
        :return:
        """
        config = ConfigParser.RawConfigParser()
        config.add_section(section)
        for key, val in self.content_dict.iteritems():
            if val is not None:
                config.set(section, key, val)

        self.content_ini = self.get_content_name() + '.ini'
        ini_path = os.path.join(self.get_content_path(), self.get_content_ini_file())
        try:
            f = open(ini_path, 'wb')
            config.write(f)
            f.close()
        except IOError:
            print ('We have a problem with writing content file %s to disc' % ini_path)
            raise

    def _create_check_script(self):
        if self.check_script:
            utils.write_to_file(os.path.join(self.get_content_path(), self.get_check_script()), 'wb', settings.temp_check_script)
            os.chmod(os.path.join(self.get_content_path(), self.get_check_script()), 0755)

    def _create_solution_file(self):
        if self.solution_file:
            utils.write_to_file(os.path.join(self.get_content_path(), self.get_solution_file()), 'wb', '')

    def _create_group_ini(self):
        """
        INI file should look like
        [preupgrade]
        group_title = <some title>
        :return:
        """
        config = ConfigParser.RawConfigParser()
        config.add_section(section)
        config.set(section, 'group_title', 'Title for %s ' % self.get_group_name())

        file_name = 'group.ini'
        group_ini = os.path.join(self.get_upgrade_path(), self.get_group_name(), file_name)
        if os.path.exists(group_ini):
            return
        try:
            f = open(group_ini, 'wb')
            config.write(f)
            f.close()
        except IOError:
            print ('We have a problem with writing %s file to disc' % group_ini)
            raise

    def create_final_content(self):
        try:
            self._create_group_ini()
            self._create_ini_file()
        except IOError:
            return None

        self._create_check_script()
        self._create_solution_file()
        return True

    def _brief_summary(self):
        content_path = os.path.join(self.upgrade_path, self.get_group_name(), self.get_content_name())
        result_content_path = os.path.join(self.upgrade_path + '-results',
                                           self.get_group_name(),
                                           self.get_content_name())
        print (settings.summary_title)
        print (settings.summary_directory % self.get_content_path())
        print (settings.summary_ini % os.path.join(content_path, self.get_content_ini_file()))
        print (settings.summary_check % os.path.join(content_path, self.get_check_script()))
        print (settings.summary_solution % os.path.join(content_path, self.get_solution_file()))
        print (settings.text_for_testing % (content_path, os.path.join(result_content_path, ALL_XCCDF_XML)))

    def take_manadatory_info(self):
        try:
            if self.specify_upgrade_path() is None:
                return 1
            self.get_content_info()
            if self.refresh_content:
                shutil.rmtree(self.get_content_path())
                os.makedirs(self.get_content_path())

            if self.create_final_content() is None:
                return 1
            self._brief_summary()
        except KeyboardInterrupt:
            print ('\n Content creation was interrupted by user.\n')
            shutil.rmtree(self.get_content_path())



# -*- coding: utf-8 -*-
"""
The application module serves for running oscap binary and reporting results to UI
"""

from __future__ import unicode_literals, print_function
import shutil
import datetime
import os
import sys
import six
from distutils import dir_util

try:
    from xmlrpclib import Fault
except ImportError:
    from xmlrpc.client import Fault

from preup import xccdf, xml_manager, remediate, utils, settings
from preup.common import Common
from preup.scanning import ScanProgress, format_rules_to_table
from preup.utils import check_xml, get_file_content, check_or_create_temp_dir
from preup.utils import run_subprocess, get_assessment_version, get_message
from preup.utils import tarball_result_dir
from preup.logger import log_message, logging, set_level
from preup.report_parser import ReportParser
from preup.kickstart.application import KickstartGenerator
from preuputils.compose import XCCDFCompose
from preup.version import VERSION


def fault_repr(self):
    """monkey patching Fault's repr method so newlines are actually interpreted"""
    log_message(self.faultString)
    return "<Fault %s: %s>" % (self.faultCode, self.faultString)

Fault.__repr__ = fault_repr


def list_contents(source_dir):
    """Function returns a list of installed contents"""
    content_dict = {}
    is_dir = lambda x: os.path.isdir(os.path.join(source_dir, x))
    dirs = os.listdir(source_dir)
    for dir_name in filter(is_dir, dirs):
        full_dir_name = os.path.join(source_dir, dir_name, settings.content_file)
        if os.path.exists(full_dir_name):
            log_message('%s' % dir_name, print_output=False)
            content_dict[dir_name] = full_dir_name

    return content_dict


def show_message(message):
    """
    Prints message out on stdout message (kind of yes/no) and return answer.

    Return values are:
    Return True on accept (y/yes). Otherwise returns False
    """
    accept = ['y', 'yes']
    choice = get_message(title=message, prompt=' y/n')
    if choice in accept:
        return True
    else:
        return False


class Application(object):

    """Class for oscap binary and reporting results to UI"""

    binary = "/usr/bin/oscap"
    command_eval = ['xccdf', 'eval']
    command_remediate = ['xccdf', 'remediate']

    def __init__(self, conf):
        """conf is preup.conf.Conf object, contains configuration"""
        self.conf = conf
        self.content = ""
        self.result_file = ""
        self.xml_mgr = None
        self.basename = ""
        self.scanning_progress = None
        self.report_parser = None
        self.third_party = ""
        self.report_data = {}
        self.text_convertor = ""
        self.common = None
        self._devel_mode = 0
        self._dist_mode = None
        if self.conf.debug is None:
            set_level(logging.INFO)
        else:
            set_level(logging.DEBUG)

    def get_command_generate(self):
        command_generate = ['xccdf', 'generate', 'report']
        return command_generate

    def get_third_party_name(self):
        """Function returns correct third party name"""
        if self.third_party != "" and not self.third_party.endswith("_"):
            self.third_party += "_"
        return self.third_party

    def get_default_xml_result_path(self):
        """Returns full XML result path"""
        return os.path.join(self.conf.result_dir,
                            self.get_third_party_name() + self.conf.xml_result_name)

    def get_default_html_result_path(self):
        """Returns full HTML result path"""
        return os.path.join(self.conf.result_dir,
                            self.get_third_party_name() + self.conf.html_result_name)

    def get_default_tarball_path(self):
        """Returns full tarball path"""
        return os.path.join(self.conf.result_dir, self.conf.tarball_name)

    def get_default_txt_result_path(self):
        """
        Function returns default txt result path based on result_dir

        :return: default txt result path
        """
        return os.path.join(self.conf.result_dir,
                            self.get_third_party_name() + self.conf.result_name + ".txt")

    def get_binary(self):
        """
        Returns oscap binary

        :return: list with path to oscap binary
        """
        return [self.binary]

    def get_preupgrade_kickstart(self):
        return settings.PREUPGRADE_KS

    def get_third_party_dir(self, assessment):
        """
        Function returns a 3rdparty dir for upgrade path
        like /root/preupgrade/RHEL6_7/3rdparty
        """
        return os.path.join(assessment, settings.add_ons)

    def get_postupgrade_dir(self):
        """Function returns postupgrade dir"""
        return os.path.join(self.conf.result_dir, settings.postupgrade_dir)

    def build_generate_command(self, xml_file, html_file):
        """Function builds a command for generating results"""
        command = self.get_binary()
        command.extend(self.get_command_generate())
        command.extend(("--output", html_file))
        command.append(check_xml(xml_file))
        return command

    def build_command(self):
        """create command from configuration"""
        self.result_file = self.get_default_xml_result_path()
        command = self.get_binary()
        report = self.get_default_html_result_path()
        command.extend(self.command_eval)
        command.append('--progress')
        command.extend(('--profile', self.conf.profile))

        # take name of content and create report: <content_name>.html
        #command.extend(('--report', report))
        command.extend(("--results", self.result_file))
        command.append(check_xml(self.content))
        return command

    def upload_results(self, tarball_path=None):
        """upload tarball with results to frontend"""
        import xmlrpclib
        import socket
        if self.conf.upload is True:
            # lets try default configuration
            url = "http://127.0.0.1:8099/submit/"
        else:
            url = self.conf.upload \
                if self.conf.upload[-1] == '/' \
                else self.conf.upload + '/'
        try:
            proxy = xmlrpclib.ServerProxy(url)
            proxy.submit.ping()
        except Exception:
            raise Exception('Can\'t connect to preupgrade assistant WEB-UI at %s.\n\n'
                'Please ensure that package preupgrade-assistant-ui '
                'has been installed on target system and firewall is set up '
                'to allow connections on port 8099.' % url)

        tarball_results = self.conf.results or tarball_path
        file_content = get_file_content(tarball_results, 'rb', False, False)

        binary = xmlrpclib.Binary(file_content)
        host = socket.gethostname()
        response = proxy.submit.submit_new({
            'data': binary,
            'host': host,
        })
        try:
            status = response['status']
        except KeyError:
            log_message('Invalid response from server.')
            log_message("Invalid response from server: %s" % response, level=logging.ERROR)
        else:
            if status == 'OK':
                try:
                    url = response['url']
                except KeyError:
                    log_message('Report submitted successfully.')
                else:
                    log_message('Report submitted successfully. You can inspect it at %s' % url)
            else:
                try:
                    message = response['message']
                    log_message('Report not submitted. Server returned message: ', message)
                    log_message("Report submit: %s (%s)" % (status, message), level=logging.ERROR)
                except KeyError:
                    log_message('Report not submitted. Server returned status: ', status)
                    log_message("Report submit: %s" % status, level=logging.ERROR)

    def apply_scan(self):
        """Extract tar ball for remediation"""
        self.prepare_apply_directories()
        tarball_result_dir(self.conf.apply,
                           self.conf.result_dir,
                           self.conf.verbose,
                           direction=False)
        if remediate.hash_postupgrade_file(self.conf.verbose, self.get_postupgrade_dir(), check=True):
            remediate.postupgrade_scripts(self.conf.verbose, self.get_postupgrade_dir())

    def prepare_scan_directories(self):
        """Used for prepartion of directories used during scan functionality"""
        self.basename = os.path.basename(self.content)
        #today = datetime.datetime.today()
        if not self.conf.temp_dir:
            check_or_create_temp_dir(self.conf.result_dir)
        check_or_create_temp_dir(self.conf.result_dir)
        check_or_create_temp_dir(settings.tarball_result_dir)
        for dir_name in settings.preupgrade_dirs:
            check_or_create_temp_dir(os.path.join(self.conf.result_dir, dir_name))

        # Copy README files into proper directories
        for key, val in six.iteritems(settings.readme_files):
            shutil.copyfile(os.path.join(settings.share_dir, "preupgrade", key),
                            os.path.join(self.conf.result_dir, val))

    def prepare_apply_directories(self):
        """Used for preparation of directories during remedation"""

        check_or_create_temp_dir(self.conf.result_dir)

    def get_total_check(self):
        """Returns a total check"""
        return self.report_parser.get_number_checks()

    def run_scan_process(self):
        """Function scans the source system"""
        self.xml_mgr = xml_manager.XmlManager(self.conf.result_dir,
                                              self.get_scenario(),
                                              os.path.basename(self.content),
                                              self.conf.result_name)

        self.report_parser.add_global_tags(self.conf.result_dir,
                                           self.get_proper_scenario(self.get_scenario()),
                                           self.conf.mode,
                                           self._devel_mode,
                                           self._dist_mode)

        self.report_parser.modify_result_path(self.conf.result_dir,
                                              self.get_proper_scenario(self.get_scenario()),
                                              self.conf.mode)
        # Execute assessment
        self.scanning_progress = ScanProgress(self.get_total_check(), self.conf.debug)
        self.scanning_progress.set_names(self.report_parser.get_name_of_checks())
        log_message('%s:' % settings.assessment_text,
                    new_line=True,
                    log=False)
        log_message('%.3d/%.3d ...running (%s)' % (
                    1,
                    self.get_total_check(),
                    self.scanning_progress.get_full_name(0)),
                    new_line=False,
                    log=False)
        start_time = datetime.datetime.now()
        self.run_scan(function=self.scanning_progress.show_progress)
        end_time = datetime.datetime.now()
        diff = end_time - start_time
        log_message(
            "Assessment finished (time %.2d:%.2ds)" % (diff.seconds/60,
                                                       diff.seconds%60),
            log=False
        )

    def run_scan(self, function=None):
        """
        The function is used for either scanning system or
        for applying changes on the target system
        """
        cmd = self.build_command()
        #log(self.conf.verbose, "running command:\n%s", ' '.join(cmd))
        # fail if openscap wasn't successful; if debug, continue
        return run_subprocess(cmd, print_output=False, function=function)

    def run_generate(self, xml_file, html_file):
        """
        The function generates result.html file from result.xml file
        which was modified by preupgrade assistant
        """
        cmd = self.build_generate_command(xml_file, html_file)
        return run_subprocess(cmd, print_output=True)

    def get_scenario(self):
        """The function returns scenario"""
        scenario = None
        try:
            sep_content = os.path.dirname(self.content).split('/')
            if self.conf.contents:
                dir_name = utils.get_valid_scenario(self.content)
                if dir_name is None:
                    return None
                check_name = dir_name
            else:
                check_name = self.conf.scan
            scenario = [x for x in sep_content if check_name in x][0]
        except IndexError:
            scenario = None
        return scenario

    def clean_preupgrade_environment(self):
        """
        Function cleans files created by preupgrade-assistant

        :return:
        """
        force_directories = [self.conf.result_dir]
        delete_directories = [settings.tarball_result_dir,
                              settings.cache_dir,
                              settings.log_dir]
        for dir_name in force_directories:
            if os.path.isdir(dir_name):
                shutil.rmtree(dir_name)
        for dir_name in delete_directories:
            for root, dirs, files in os.walk(dir_name, topdown=False):
                for name in files:
                    os.remove(os.path.join(root, name))
                for name in dirs:
                    shutil.rmtree(os.path.join(root, name))

    def clean_scan(self):
        """
        The function remove symlink /root/preupgrade from older versions
        Also it removes directory /root/preupgrade because of new assessment.
        """
        if os.path.islink(self.conf.result_dir):
            os.unlink(self.conf.result_dir)
        if os.path.isdir(self.conf.result_dir):
            shutil.rmtree(self.conf.result_dir)

    def prepare_for_generation(self):
        """Function prepares the XML file for conversion to HTML format"""
        for report in self._get_reports():
            ReportParser.write_xccdf_version(report, direction=True)
            self.run_generate(report, report.replace('.xml', '.html'))
            # Switching back namespace
            ReportParser.write_xccdf_version(report)

    def prepare_xml_for_html(self):
        """The function prepares a XML file for HTML creation"""
        # Reload XML file
        self.report_parser.reload_xml(self.get_default_xml_result_path())
        # Replace fail in case of none or slight risk with needs_inspection
        self.report_parser.replace_inplace_risk(scanning_results=self.scanning_progress)
        if not self.conf.debug:
            self.report_parser.remove_debug_info()
        self.report_parser.reload_xml(self.get_default_xml_result_path())
        self.report_parser.update_check_description()
        self.prepare_for_generation()

        if not self.conf.verbose:
            self.xml_mgr.remove_html_information()
        # This function finalize XML operations
        self.finalize_xml_files()
        if self.conf.text:
            run_subprocess(self.get_cmd_convertor(),
                           print_output=False,
                           shell=True)

    def _get_reports(self):
        reports = [self.get_default_xml_result_path()]
        report_admin = self.report_parser.get_report_type(settings.REPORTS[0])
        if report_admin:
            reports.append(report_admin)
        # We separate user contents
        report_user = self.report_parser.get_report_type(settings.REPORTS[1])
        if report_user:
            reports.append(report_user)
        return reports

    def finalize_xml_files(self):
        """
        Function copies postupgrade scripts and creates hash postupgrade file.
        It finds solution files and update XML file.
        """
        # Copy postupgrade.d special files
        remediate.special_postupgrade_scripts(self.conf.result_dir)
        remediate.hash_postupgrade_file(self.conf.verbose,
                                        self.get_postupgrade_dir())
        solution_files = self.report_parser.get_solution_files()
        for report in self._get_reports():
            self.xml_mgr.find_solution_files(report.split('.')[0], solution_files)
        remediate.copy_modified_config_files(self.conf.result_dir)

    def run_third_party_modules(self, dir_name):
        """
        Functions executes a 3rd party contents

        3rd party contents are stored in
        /usr/share/preupgrade/RHEL6_7/3rdparty directory
        """
        for self.third_party, content in six.iteritems(list_contents(dir_name)):
            third_party_name = self.third_party
            log_message("Execution {0} assessments:".format(self.third_party))
            self.report_parser.reload_xml(content)
            self.content = content
            self.run_scan_process()
            self.report_data[third_party_name] = self.scanning_progress.get_output_data()
            # This function prepare XML and generate HTML
            self.prepare_xml_for_html()

        self.third_party = ""

    def get_cmd_convertor(self):
        """Function returns cmd with text convertor string"""
        cmd = settings.text_converters[self.text_convertor].format(
            self.text_convertor,
            self.get_default_html_result_path(),
            self.get_default_txt_result_path()
        )
        return cmd

    def get_proper_scenario(self, scenario):
        if not self.conf.contents:
            return scenario
        scenario = scenario.replace('-results', '')
        return scenario

    def prepare_scan_system(self):
        """Function cleans previous scan and creates relevant directories"""
        # First of all we need to delete the older one assessment
        self.clean_scan()
        self.prepare_scan_directories()
        scenario = self.get_scenario()
        if scenario is None:
            log_message('Invalid scenario: %s' % self.conf.contents)
            sys.exit(3)
        scenario_path = os.path.join(self.conf.source_dir, scenario)
        if not os.path.isdir(scenario_path):
            log_message('Invalid scenario: %s' % scenario,
                        level=logging.ERROR)
            sys.exit(3)

    def generate_report(self):
        """Function generates report"""
        scenario = self.get_scenario()
        scenario_path = os.path.join(self.conf.source_dir, scenario)
        assessment_dir = os.path.join(self.conf.result_dir,
                                      self.get_proper_scenario(scenario))
        dir_util.copy_tree(scenario_path, assessment_dir)
        # Try copy directory with contents to /root/preupgrade
        # Call xccdf_compose API for generating all-xccdf.xml
        if not self.conf.contents:
            xccdf_compose = XCCDFCompose(assessment_dir)
            generated_dir = xccdf_compose.generate_xml(generate_from_ini=False)
            if os.path.isdir(assessment_dir):
                shutil.rmtree(assessment_dir)
            shutil.move(generated_dir, assessment_dir)

        self.common.prep_symlinks(assessment_dir,
                                  scenario=self.get_proper_scenario(scenario))
        if not self.conf.contents:
            utils.update_platform(os.path.join(assessment_dir, settings.content_file))
        else:
            utils.update_platform(self.content)
            assessment_dir = os.path.dirname(self.content)
        return assessment_dir

    def copy_preupgrade_scripts(self, assessment_dir):
        # Copy preupgrade-scripts directory from scenarvirtuio
        preupg_scripts = os.path.join(assessment_dir, settings.preupgrade_name)
        if os.path.exists(preupg_scripts):
            dir_util.copy_tree(preupg_scripts, settings.preupgrade_scripts)

    def scan_system(self):
        """The function is used for scanning system with all steps."""
        self._set_devel_mode()
        self.prepare_scan_system()
        assessment_dir = self.generate_report()
        # Update source XML file in temporary directory
        self.content = os.path.join(assessment_dir, settings.content_file)
        try:
            self.report_parser = ReportParser(self.content)
        except IOError:
            log_message("Content {0} does not exist".format(self.content))
            sys.exit(1)
        if not self.conf.contents:
            version = get_assessment_version(self.conf.scan)
            if version is None:
                log_message("Your scan have wrong format",
                            level=logging.ERROR)
                log_message("Examples format is like RHEL6_7",
                            level=logging.ERROR)
                sys.exit(1)
            self.report_parser.modify_platform_tag(version[0])
        if self.conf.mode:
            try:
                lines = [i.rstrip() for i in get_file_content(os.path.join(os.path.dirname(self.path),
                                                                           self.conf.mode),
                                                              'rb',
                                                              method=True)]
            except IOError:
                return
            self.report_parser.select_rules(lines)
        if self.conf.select_rules:
            lines = [i.strip() for i in self.conf.select_rules.split(',')]
            unknown_rules = self.report_parser.check_rules(lines)
            if unknown_rules:
                log_message(settings.unknown_rules % '\n'.join(unknown_rules))
            self.report_parser.select_rules(lines)
        self.run_scan_process()
        main_report = self.scanning_progress.get_output_data()
        # This function prepare XML and generate HTML
        self.prepare_xml_for_html()

        third_party_dir_name = self.get_third_party_dir(assessment_dir)
        if os.path.exists(third_party_dir_name):
            self.run_third_party_modules(third_party_dir_name)

        self.copy_preupgrade_scripts(assessment_dir)

        # It prints out result in table format
        format_rules_to_table(main_report, "main contents")
        for target, report in six.iteritems(self.report_data):
            format_rules_to_table(report, "3rdparty content " + target)

        tar_ball_name = tarball_result_dir(self.conf.tarball_name, self.conf.result_dir, self.conf.verbose)
        log_message("Tarball with results is stored here '%s' ." % tar_ball_name)
        log_message("The latest assessment is stored in directory '%s' ." % self.conf.result_dir)
        # pack all configuration files to tarball
        return tar_ball_name

    def summary_report(self, tarball_path):
        """Function prints a summary report"""
        command = settings.ui_command.format(tarball_path)
        if self.conf.text:
            path = self.get_default_txt_result_path()
        else:
            path = self.get_default_html_result_path()

        report_dict = {
            0: settings.message.format(path),
            1: settings.message.format(path),
            2: 'We found some critical issues. In-place upgrade is not advised.\n' +
            "Read the file {0} for more details.".
            format(path)
        }
        return_value = xccdf.check_inplace_risk(self.get_default_xml_result_path(), 0)
        try:
            if report_dict[int(return_value)]:
                log_message('Summary information:')
                log_message(report_dict[int(return_value)])
            for report_type in settings.REPORTS:
                file_name = settings.result_name + '-' + report_type + '.html'
                report_name = os.path.join(os.path.dirname(self.report_parser.get_path()), file_name)
                if os.path.exists(report_name):
                    log_message("Read the %s report file %s for more details." % (report_type, report_name))
        except KeyError:
            # We do not want to print anything in case of testing contents
            pass
        if self.report_data:
            log_message('Summary 3rd party providers:')
            for target, dummy_report in six.iteritems(self.report_data):
                self.third_party = target
                log_message("Read the 3rd party content {0} {1} for more details.".
                            format(target, path))
        log_message("Upload results to UI by command:\ne.g. {0} .".format(command))

    def _set_devel_mode(self):
        # Check for devel_mode
        if os.path.exists(settings.DEVEL_MODE):
            self._devel_mode = 1
            self._dist_mode = utils.get_preupg_config_file(settings.PREUPG_CONFIG_FILE,
                                                           'dist_mode',
                                                           section="devel-mode")
        else:
            self._devel_mode = 0

    def run(self):
        """run analysis"""
        if self.conf.version:
            print ("Preupgrade Assistant version: %s" % VERSION)
            return 0

        if not self.conf.scan and not self.conf.contents:
            cnt = 0
            is_dir = lambda x: os.path.isdir(os.path.join(self.conf.source_dir, x))
            dirs = os.listdir(self.conf.source_dir)
            for dir_name in filter(is_dir, dirs):
                if utils.get_assessment_version(dir_name):
                    self.conf.scan = dir_name
                    cnt += 1

            if int(cnt) < 1:
                log_message("There were no contents found in directory %s. \
If you would like to use this tool, you have to install some." % settings.source_dir)
                return 1
            if int(cnt) > 1:
                log_message("Preupgrade assistant detects more then 1 set of contents in directory%s. \
If you would like to use this tool, you have to specify correct upgrade path parameter like -s RHEL6_7." % settings.source_dir)
                return 1

        if self.conf.list_contents_set:
            for dir_name, dummy_content in six.iteritems(list_contents(self.conf.source_dir)):
                log_message("{0}".format(dir_name))
            return 0

        if self.conf.list_rules:
            log_message(settings.list_rules % '\n'.join(utils.get_list_rules(self.conf.scan)))
            return 0

        if self.conf.upload and self.conf.results:
            self.upload_results()
            return 0

        if self.conf.mode and self.conf.select_rules:
            log_message(settings.options_not_allowed)
            return 1

        if not self.conf.riskcheck and not self.conf.apply and not self.conf.cleanup and not self.conf.kickstart:
            # If force option is not mentioned and user select NO then exits
            if not self.conf.force and not show_message(settings.warning_text):
                # We do not want to continue
                return 0

        if self.conf.text:
            # Test whether w3m, lynx and elinks packages are installed
            found = False
            for pkg in utils.get_convertors():
                if xml_manager.get_package_version(pkg):
                    self.text_convertor = pkg
                    found = True
                    break
            if not found:
                log_message(settings.converter_message.format(' '.join(utils.get_convertors())))
                return 0

        if os.geteuid() != 0:
            print("Need to be root", end="\n")
            if not self.conf.debug:
                return 2

        if self.conf.cleanup:
            self.clean_preupgrade_environment()
            sys.exit(0)

        if self.conf.riskcheck:
            return_val = xccdf.check_inplace_risk(self.get_default_xml_result_path(), self.conf.verbose)
            return return_val

        if self.conf.kickstart:
            if not os.path.exists(self.get_default_xml_result_path()):
                log_message("'preupg' command was not run yet. Run them before kickstart generation.")
                return 1
            kg = KickstartGenerator(self.conf, settings.KS_DIR, self.get_preupgrade_kickstart())
            kg.main()
            return 0

        if self.conf.scan:
            self.content = os.path.join(self.conf.source_dir,
                                        self.conf.scan,
                                        settings.content_file)
            if self.conf.scan.startswith("/"):
                log_message('Specify correct upgrade path parameter like -s RHEL6_7')
                log_message('Upgrade path is provided by command preupg --list')
                return 1
            if not os.path.isdir(os.path.join(self.conf.source_dir, self.conf.scan)):
                log_message('Specify correct upgrade path parameter like -s RHEL6_7')
                log_message('Upgrade path is provided by command preupg --list')
                return 1

        if self.conf.contents:
            self.content = os.path.join(os.getcwd(), self.conf.contents)
            # From content path like content-users/RHEL6_7 we need
            # to get content-users dir
            content_dir = self.conf.contents[:self.conf.contents.find(self.get_scenario())]
            self.conf.source_dir = os.path.join(os.getcwd(), content_dir)

        self.common = Common(self.conf)
        if not self.conf.skip_common:
            if not self.common.common_results():
                return 1

        if self.conf.scan or self.conf.contents:
            if not os.path.exists(self.binary):
                log_message("Oscap with SCE enabled is not installed")
                return 1
            if not os.access(self.binary, os.X_OK):
                log_message("Oscap with SCE %s is not executable" % self.binary)
                return 1

            current_dir = os.getcwd()
            os.chdir("/tmp")
            tarball_path = self.scan_system()
            self.summary_report(tarball_path)
            self.common.copy_common_files()
            KickstartGenerator.kickstart_scripts()
            utils.remove_home_issues()
            if self.conf.upload:
                self.upload_results(tarball_path)
            os.chdir(current_dir)
            return 0

        log_message('Nothing to do. Give me a task, please.')
        self.conf.settings[2].parser.print_help()
        return 0



# -*- coding: utf-8 -*-

from __future__ import unicode_literals
import re
import six
from operator import itemgetter
from xml.etree import ElementTree

from preup import utils
from preup.logger import log_message

XMLNS = "{http://checklists.nist.gov/xccdf/1.2}"


def list_groups(xccdf_file):
    """
    list groups of tests from provided xccdf file
    """
    return []


def get_and_print_inplace_risk(verbose, inplace_risk):
    """
    The function browse throw the list and find first
    inplace_risk and return corresponding status.
    If verbose mode is used then it prints out
    all inplace risks higher then SLIGHT.
    """
    risks = {
        'NONE:': 0,
        'SLIGHT:': 1,
        'MEDIUM:': 2,
        'HIGH:': 3,
        'EXTREME:': 4,
    }

    return_value = -1
    for key, val in sorted(six.iteritems(risks), key=itemgetter(1), reverse=False):
        matched = [x for x in inplace_risk if key in x]
        if matched:
            # if matched and return_value the remember her
            if return_value < val:
                return_value = val
            # If verbose mode is used and value is bigger then 0 then prints out
            if int(verbose) > 1:
                log_message('\n'.join(matched), print_output=verbose)
            elif int(verbose) == 1 and val > 0:
                log_message('\n'.join(matched), print_output=verbose)

    return return_value


def get_check_import_inplace_risk(tree):
    """
    Function returns implace risks
    """
    inplace_risk = []
    risk_regex = "INPLACERISK: (?P<level>\w+): (?P<message>.+)"
    for check in tree.findall(".//" + XMLNS + "check-import"):
        if not check.text:
            continue
        lines = check.text.strip().split('\n')
        for line in lines:
            match = re.match(risk_regex, line)
            if match:
                inplace_risk.append(line)
    return inplace_risk


def check_inplace_risk(xccdf_file, verbose):
    """
    The function read the content of the file
    and finds out all INPLACERISK rows in TestResult tree.
    return value is get from function get_and_print_inplace_risk
    """
    try:
        content = utils.get_file_content(xccdf_file, 'rb', False, False)
        if not content:
            # WE NEED TO RETURN -1 FOR RED-HAT-UPGRADE-TOOL
            return -1
    except IOError:
        # WE NEED TO RETURN -1 FOR RED-HAT-UPGRADE-TOOL
        return -1

    inplace_risk = []
    target_tree = ElementTree.fromstring(content)
    for profile in target_tree.findall(XMLNS + "TestResult"):
        inplace_risk = get_check_import_inplace_risk(profile)

    result = get_and_print_inplace_risk(verbose, inplace_risk)
    # different behaviour of division between py2 & 3
    if(result == -1):
        return -1
    elif(result < 2):
        return 0
    elif(result < 4):
        return 1
    else:
        return 2



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals
import six
import datetime
import re
import subprocess
import fnmatch
import os
import sys
import shutil
import ConfigParser

from preup import settings
from preup.logger import log_message, logging

from os import path, access, W_OK, R_OK, X_OK


def check_file(fp, mode):
    """
    Check if file exists and has set right mode

    mode can be in string format as for function open (available letters: wrax)
    or int number (in that case prefered are os constants W_OK, R_OK, X_OK)
    (letter 'a' has same signification as 'w', is here due to compatibility
    with open mode)
    """
    intern_mode = 0
    if(isinstance(mode, six.text_type)):
        if('w' in mode or 'a' in mode):
            intern_mode += W_OK
        if('r' in mode):
            intern_mode += R_OK
        if('x' in mode):
            intern_mode += X_OK
    else:
        intern_mode = mode
    if(path.exists(fp)):
        if(path.isfile(fp)):
            if(access(fp, intern_mode)):
                return True
            else:
                return False
        else:
            return False
    else:
        return False


def check_xml(xml_file):
    """
    Check XML

    return False if xml file is not okay or raise IOError if perms are
    not okay; use python-magic to check the file if module is available
    """
    if os.path.isfile(xml_file):
        if not os.access(xml_file, os.R_OK):
            log_message("File is not readable." % xml_file, level=logging.ERROR)
            raise IOError("File %s is not readable." % xml_file)
    else:
        log_message("%s is not a file" % xml_file, level=logging.ERROR)
        raise IOError("%s is not a file." % xml_file)
    raw_test = False
    is_valid = False
    try:
        import magic
    except ImportError:
        raw_test = True
    else:
        try:
            xml_file_magic = magic.from_file(xml_file, mime=True)
        except AttributeError:
            raw_test = True
        else:
            is_valid = xml_file_magic == 'application/xml'
    if raw_test:
        is_valid = xml_file.endswith(".xml")
    if is_valid:
        return xml_file
    else:
        log_message("Provided file is not a valid XML file", level=logging.ERROR)
        raise RuntimeError("Provided file is not a valid XML file")


def check_or_create_temp_dir(temp_dir, mode=None):
    """Check if provided temp dir is valid."""
    if os.path.isdir(temp_dir):
        if not os.access(temp_dir, os.W_OK):
            log_message("Directory %s is not writable." % temp_dir, level=logging.ERROR)
            raise IOError("Directory %s is not writable." % temp_dir)
    else:
        os.makedirs(temp_dir)
    if mode:
        os.chmod(temp_dir, mode)
    return temp_dir


def get_interpreter(filename, verbose=False):
    """
    The function returns interpreter

    Checks extension of script and first line of script
    """
    script_types = {'/bin/bash': '.sh',
                    '/usr/bin/python': '.py',
                    '/usr/bin/perl': '.pl'}
    inter = list(k for k, v in six.iteritems(script_types) if filename.endswith(v))
    content = get_file_content(filename, 'rb')
    if inter and content.startswith('#!'+inter[0]):
        return inter
    else:
        if verbose:
            log_message("Problem with getting interpreter", level=logging.ERROR)
        return None


def print_error_msg(title="", msg="", level=' ERROR '):
    """Function prints a ERROR or WARNING messages"""
    number = 10
    print ('\n')
    print ('*'*number+level+'*'*number)
    print (title, ''.join(msg))


def run_subprocess(cmd, output=None, print_output=False, shell=False, function=None):
    """wrapper for Popen"""
    sp = subprocess.Popen(cmd,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.STDOUT,
                          shell=shell,
                          bufsize=1)
    stdout = six.binary_type() # FIXME should't be this bytes()?
    for stdout_data in iter(sp.stdout.readline, b''):
        # communicate() method buffers everything in memory, we will read stdout directly
        stdout += stdout_data
        if function is None:
            if print_output:
                print (stdout_data, end="")
        else:
            # I don't know what functions can come here, however
            # it's not common so put only unicode data here again.
            # Should be always raw data so we don't need test stdout_data
            # on type
            function(stdout_data.decode(settings.defenc))
    sp.communicate()

    if output is not None:
        # raw data, so without encoding
        write_to_file(output, "wb", stdout, False)
    return sp.returncode


def create_dest_dir(path):
    n = datetime.datetime.now()
    stamp = n.strftime("%y%m%d%H%M%S%f")
    if path.endswith('/'):
        destdir = path[:-1] + stamp
    else:
        destdir = path + stamp
    os.makedirs(destdir)
    return destdir


def get_prefix():
    return settings.prefix


def get_system():
    """
    Check if system is Fedora or RHEL

    :return: Fedora or None
    """
    lines = get_file_content('/etc/redhat-release', 'rb', method=True)
    return [line for line in lines if line.startswith('Fedora')]


def get_assessment_version(dir_name):
    if get_prefix() == "preupgrade":
        matched = re.search(r'\D+(\d*)_(\d+)', dir_name, re.I)
        if matched:
            return [matched.group(1), matched.group(2)]
        else:
            return None
    elif get_prefix() == "premigrate":
        matched = re.search(r'\D+(\d*)_\D+(\d+)', dir_name, re.I)
        if matched:
            return [matched.group(1), matched.group(2)]
        else:
            return None
    else:
        matched = re.search(r'\D+(\d*)_(\D*)(\d+)', dir_name, re.I)
        if matched:
            return [matched.group(1), matched.group(3)]
        else:
            return None


def get_valid_scenario(dir_name):
    matched = [x for x in dir_name.split(os.path.sep) if re.match(r'\D+(\d*)_(\D*)(\d+)(-results)?$', x, re.I)]
    if matched:
        return matched[0]
    else:
        return None


def get_file_content(path, perms, method=False, decode_flag=True):
    """
    shortcut for returning content of file

    open(...).read()...
    if method is False then file is read by function read
    if method is True then file is read by function readlines
    When decode_flag is True, read string is decoded to unicode. Otherwise
    only read. (Some libraries request non-unicode strings - as ElementTree)
    """

    # data must be init due to possible troubles with binary data
    data = None
    try:
        f = open(path, perms)
        try:
            if decode_flag is True:
                data = f.read().decode(settings.defenc) if not method else [line.decode(settings.defenc) for line in f.readlines()]
            else:
                data = f.read() if not method else f.readlines()
        finally:
            f.close()
    except IOError:
        raise
    if data is None:
        raise ValueError("You try decode binary data to unicode: %s" % path)
    return data


def write_to_file(path, perms, data, encode_flag=True):
    """
    shortcut for write of data to file:

    open(...).write()...
    data can be string or list of strings

    data contains unicode string(s) in most cases, so we encode them
    to system default encoding before write. When you use encoded strings,
    set encode_flag to False to suppress second encodiding process.
    """
    try:
        f = open(path, perms)
        try:
            if isinstance(data, list):
                if encode_flag is True:
                    data = [line.encode(settings.defenc) for line in data]
                f.writelines(data)
            else:
                # TODO: May we should print warn w
                if encode_flag is True and isinstance(data, six.text_type):
                    f.write(data.encode(settings.defenc))
                else:
                    f.write(data)
        finally:
            f.close()
    except IOError:
        raise


def get_tarball_name(result_file, time):
    return result_file.format(time)


def get_tarball_result_path(root_dir, filename):
    return os.path.join(root_dir, filename)


def get_current_time():
    return datetime.datetime.now().strftime("%y%m%d%H%M%S")


def tarball_result_dir(result_file, dirname, quiet, direction=True):
    """
    pack results to tarball

    direction is used as a flag for packing or extracting
    For packing True
    For unpacking False
    """
    current_dir = os.getcwd()
    tar_binary = "/bin/tar"
    current_time = get_current_time()
    cmd_extract = "-xzf"
    cmd_pack = "-czvf"
    cmd = [tar_binary]
    # numeric UIDs and GIDs are used, ACLs are enabled, SELinux is enabled
    tar_options = ["--numeric-owner", "--acls", "--selinux"]

    # used for packing directories into tarball
    os.chdir('/root')
    tarball_dir = get_tarball_name(result_file, current_time)
    tarball_name = tarball_dir + '.tar.gz'
    bkp_tar_dir = os.path.join('/root', tarball_dir)
    if direction:
        shutil.copytree(dirname, bkp_tar_dir, symlinks=True)
        tarball = get_tarball_result_path(dirname, tarball_name)
        cmd.append(cmd_pack)
        cmd.append(tarball)
        cmd.append(tarball_dir)
    else:
        cmd.append(cmd_extract)
        cmd.append(result_file)

    cmd.extend(tar_options)
    run_subprocess(cmd, print_output=quiet)
    shutil.rmtree(bkp_tar_dir)
    if direction:
        try:
            shutil.copy(tarball, os.path.join(settings.tarball_result_dir+"/"))
        except IOError:
            log_message("Problem with copying tarball {0} to /root/preupgrade-results".format(tarball))
    os.chdir(current_dir)

    return os.path.join(settings.tarball_result_dir, tarball_name)


def get_upgrade_dir_path(dirname):
    """
    The function returns upgrade path dir like RHEL6_7

    If /root/preupgrade/ dir contaings RHEL6_7 dir then
    it return just RHEL6_7 dir.
    This is used for get_assessment_version
    """
    is_dir = lambda x: os.path.isdir(os.path.join(dirname, x))
    dirs = os.listdir(dirname)
    for d in filter(is_dir, dirs):
        upgrade_path = filter(lambda x: d in x, settings.preupgrade_dirs)
        if not upgrade_path:
            return d
    return None


def get_message(title="", message="Do you want to continue?", prompt=None):
    """
    Function asks for input from user

    :param title: Title of the message
    :param message: Message text
    :return: y or n
    """
    yes = ['yes', 'y', 'Y', 'Yes']
    yesno = yes + ['no', 'n', 'N', 'No']
    print (title)
    if prompt is not None:
        print (message + prompt)
    else:
        print (message)
    while True:
        try:
            if(sys.version_info[0] == 2):
                choice = raw_input()
            else:
                choice = input()
        except KeyboardInterrupt:
            raise
        if prompt and choice not in yesno:
            print ('You have to choose one of y/n.')
        else:
            return choice


def get_needs_inspection():
    return settings.needs_inspection


def get_needs_action():
    return settings.needs_action


def get_convertors():
    """Function returns list of supported convertors"""
    return settings.text_converters.keys()


def get_variant():
    """Function return a variant"""
    redhat_release = get_file_content("/etc/redhat-release", "rb")
    if redhat_release.startswith('Fedora'):
        return None
    try:
        rel = redhat_release.split()
        return rel[4]
    except IndexError:
        return None


def get_addon_variant():
    """
    Function returns a addons variant if available

    83 - HighAvailability
    85 - LoadBalancer
    90 - ResilientStorage
    92 - ScalableFileSystem
    """
    mapping_dict = {
        '83.pem': 'HighAvailability',
        '85.pem': 'LoadBalancer',
        '90.pem': 'ResilientStorage',
        '92.pem': 'ScalableFileSystem',
    }
    variant = ['optional']
    pki_dir = "/etc/pki/product"
    if os.path.isdir(pki_dir):
        pem_files = [x for x in os.listdir(pki_dir) if x.endswith(".pem")]
        for pem in pem_files:
            # Curently we don't use the openssl command for getting certificate
            # PEM numbers are not changed between versions.
            #cmd = settings.openssl_command.format(os.path.join(pki_dir, pem))
            #lines = run_subprocess(cmd, print_output=True, shell=True)
            if pem in mapping_dict.keys():
                variant.append(mapping_dict[pem])
    return variant


def clean_directory(dir_name, pattern):
    """
    Function deleted specific files in dir_name

    :param dir_name: Dirname where the files are deleted
    :param pattern: What files with specific pattern are deleted
    :return:
    """
    for root, dummy_dirs, files in os.walk(dir_name):
        for f in files:
            if fnmatch.fnmatch(f, pattern):
                os.unlink(os.path.join(root, f))


def remove_home_issues():
    """
    Function removes /home rows from specific files

    :return:
    """
    files = [os.path.join(settings.cache_dir, settings.common_name, 'allmyfiles.log'),
             os.path.join(settings.KS_DIR, 'untrackeduser')]
    for f in files:
        try:
            lines = get_file_content(f, 'rb', method=True)
            lines = [l for l in lines if not l.startswith('/home')]
            write_to_file(f, 'wb', lines)
        except IOError:
            pass


def update_platform(full_path):
    file_lines = get_file_content(full_path, 'rb', method=True)
    platform = ''
    platform_id = ''
    if not get_system():
        platform = settings.CPE_RHEL
    else:
        platform = settings.CPE_FEDORA
    platform_id = get_assessment_version(full_path)
    for index, line in enumerate(file_lines):
        if 'PLATFORM_NAME' in line:
            line = line.replace('PLATFORM_NAME', platform)
        if 'PLATFORM_ID' in line:
            line = line.replace('PLATFORM_ID', platform_id[0])
        file_lines[index] = line
    write_to_file(full_path, 'wb', file_lines)


def get_preupg_config_file(path, key, section="preupgrade"):
    if not os.path.exists(path):
        return None

    config = ConfigParser.RawConfigParser(allow_no_value=True)
    config.read(path)
    section = 'preupgrade-assistant'
    if config.has_section(section):
        if config.has_option(section, key):
            return config.get(section, key)


def get_list_rules(scenario):
    main_dir = os.path.join(settings.source_dir, scenario)
    rules = get_file_content(os.path.join(main_dir, settings.file_list_rules), "rb", method=True)
    rules = [x.strip() for x in rules]
    return rules



from __future__ import unicode_literals
import unittest
import tempfile
import shutil
import os

from preup import xccdf
from preup import utils
from preup.conf import Conf, DummyConf
from preup.cli import CLI
from preup.application import Application
from preup import settings

import base

class TestRiskCheck(base.TestCase):

    def _generate_result(self, content_temp):
        conf = {
            "contents": content_temp,
            "profile": "xccdf_preupg_profile_default",
            "result_dir": os.path.dirname(content_temp),
            "skip_common": True,
            "temp_dir": os.path.dirname(content_temp),
            "id": None,
            "debug": True,  # so root check won't fail
        }

        dc = DummyConf(**conf)
        cli = CLI(["--contents", content_temp])
        a = Application(Conf(dc, settings, cli))
        # Prepare all variables for test
        a.conf.source_dir = os.getcwd()
        a.content = a.conf.contents
        a.basename = os.path.basename(a.content)
        self.assertEqual(a.run_scan(), 0)

    def _copy_xccdf_file(self, update_text):
        temp_dir = tempfile.mkdtemp()
        xccdf_file = os.path.join(os.getcwd(), 'tests', 'FOOBAR6_7', 'dummy_preupg', 'all-xccdf.xml')
        temp_file = os.path.join(temp_dir, 'all_xccdf.xml')
        shutil.copyfile(xccdf_file, temp_file)
        content = utils.get_file_content(temp_file, 'rb', decode_flag=False)
        content = content.replace(b'INPLACE_TAG', update_text)
        utils.write_to_file(temp_file, 'wb', content)
        return temp_file

    def test_check_inplace_risk_high(self):

        temp_file = self._copy_xccdf_file(b'INPLACERISK: HIGH: Test High Inplace risk')
        self._generate_result(temp_file)
        return_value = xccdf.check_inplace_risk(os.path.join(os.path.dirname(temp_file), 'result.xml'), 0)
        shutil.rmtree(os.path.dirname(temp_file))
        self.assertEqual(return_value, 1)

    def test_check_inplace_risk_medium(self):

        temp_file = self._copy_xccdf_file(b'INPLACERISK: MEDIUM: Test Medium Inplace risk')
        self._generate_result(temp_file)
        return_value = xccdf.check_inplace_risk(os.path.join(os.path.dirname(temp_file), 'result.xml'), 0)
        shutil.rmtree(os.path.dirname(temp_file))
        self.assertEqual(return_value, 1)

    def test_check_inplace_risk_slight(self):

        temp_file = self._copy_xccdf_file(b'INPLACERISK: SLIGHT: Test Slight Inplace risk')
        self._generate_result(temp_file)
        return_value = xccdf.check_inplace_risk(os.path.join(os.path.dirname(temp_file), 'result.xml'), 0)
        shutil.rmtree(os.path.dirname(temp_file))
        self.assertEqual(return_value, 0)

    def test_check_inplace_risk_none(self):

        temp_file = self._copy_xccdf_file(b'INPLACERISK: NONE: Test None Inplace risk')
        self._generate_result(temp_file)
        return_value = xccdf.check_inplace_risk(os.path.join(os.path.dirname(temp_file), 'result.xml'), 0)
        shutil.rmtree(os.path.dirname(temp_file))
        self.assertEqual(return_value, 0)

    def test_check_inplace_risk_extreme(self):

        temp_file = self._copy_xccdf_file(b'INPLACERISK: EXTREME: Test Extreme Inplace risk')
        self._generate_result(temp_file)
        return_value = xccdf.check_inplace_risk(os.path.join(os.path.dirname(temp_file), 'result.xml'), 0)
        shutil.rmtree(os.path.dirname(temp_file))
        self.assertEqual(return_value, 2)

    def test_check_inplace_risk_unknown(self):

        temp_file = self._copy_xccdf_file(b'INPLACERISK: UNKNOWN: Test Extreme Inplace risk')
        self._generate_result(temp_file)
        return_value = xccdf.check_inplace_risk(os.path.join(os.path.dirname(temp_file), 'result.xml'), 0)
        shutil.rmtree(os.path.dirname(temp_file))
        self.assertEqual(return_value, -1)


def suite():
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTest(loader.loadTestsFromTestCase(TestRiskCheck))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=3).run(suite())



from __future__ import print_function, unicode_literals
import os
import sys
import re
import datetime
import shutil

import six
from distutils import dir_util

from preup.utils import get_valid_scenario, write_to_file
from preuputils import variables
from preuputils.oscap_group_xml import OscapGroupXml
from preup import settings
from preup import xccdf
from xml.etree import ElementTree
try:
    from xml.etree.ElementTree import ParseError
except ImportError:
    from xml.parsers.expat import ExpatError as ParseError

XCCDF_FRAGMENT = "{http://fedorahosted.org/sce-community-content/wiki/XCCDF-fragment}"
SCE = "http://open-scap.org/page/SCE"


class XCCDFCompose(object):
    dir_name = ""
    result_dir = ""

    def __init__(self, argument):
        """
        Specify dirname with the on content
        :param argument: dirname where content is
        :return:
        """
        self.result_dir = argument
        self.dir_name = self.result_dir + variables.result_prefix
        if self.result_dir.endswith("/"):
            self.dir_name = self.result_dir[:-1] + variables.result_prefix

        # Delete previous contents if they exist.
        if os.path.exists(self.dir_name):
            shutil.rmtree(self.dir_name)

        if get_valid_scenario(self.dir_name) is None:
            print ('Use valid scenario like RHEL6_7 or CENTOS6_RHEL6')
            sys.exit(1)

    def generate_xml(self, generate_from_ini=True):
        dir_util.copy_tree(self.result_dir, self.dir_name)
        result_dirname = self.dir_name
        template_file = ComposeXML.get_template_file()
        try:
            target_tree = ElementTree.parse(template_file).getroot()
        except IOError:
            print ('Problem with reading template.xml file')
            sys.exit(1)
        target_tree = ComposeXML.run_compose(target_tree, self.dir_name, generate_from_ini=generate_from_ini)

        report_filename = os.path.join(result_dirname, settings.content_file)
        try:
            write_to_file(report_filename, "wb",
                          ElementTree.tostring(target_tree, "utf-8"),
                          False)
            if generate_from_ini:
                print ('Generate report file for preupgrade-assistant is:', ''.join(report_filename))
        except IOError as e:
            print ("Problem with writing file ", report_filename)
            raise
        return self.dir_name


class ComposeXML(object):

    upgrade_path = ""

    @classmethod
    def collect_group_xmls(cls, source_dir, content=None, level=0, generate_from_ini=True):
        ret = {}

        for dirname in os.listdir(source_dir):
            if content and dirname != content:
                continue
            if dirname and dirname[0] == '.':
                continue
            new_dir = os.path.join(source_dir, dirname)
            if not os.path.isdir(new_dir):
                continue
            ini_files = filter(lambda x: x.endswith(".ini"), os.listdir(new_dir))
            if not ini_files and generate_from_ini:
                # Check if directory contains only subdirectories.
                # Report to user that group.ini file could be missing
                directories = [x for x in os.listdir(new_dir) if not os.path.isdir(os.path.join(new_dir, x))]
                if not directories and 'postupgrade.d' not in dirname:
                    print ("WARNING: It seems that group.ini file is missing %s. Please check if it is really missing." % new_dir)
            if ini_files and generate_from_ini:
                oscap_group = OscapGroupXml(new_dir)
                oscap_group.write_xml()
                return_list = oscap_group.collect_group_xmls()
                cls.perform_autoqa(new_dir, return_list)

            group_file_path = os.path.join(new_dir, "group.xml")
            if not os.path.isfile(group_file_path):
                # print("Directory '%s' is missing a group.xml file!" % (new_dir))
                continue
            try:
                ret[dirname] = (ElementTree.parse(group_file_path).getroot(),
                                cls.collect_group_xmls(new_dir, level=level + 1, generate_from_ini=generate_from_ini))
            except ParseError as e:
                print ("Encountered a parse error in file ", group_file_path, " details: ", e)
        return ret

    @classmethod
    def perform_autoqa(cls, path_prefix, group_tree):
        for f, t in six.iteritems(group_tree):
            b_subgroups = True
            try:
                tree, subgroups = t
            except ValueError:
                tree = t
                b_subgroups = False

            group_xml_path = os.path.join(f, "group.xml")

            groups = tree.findall(xccdf.XMLNS + "Group")
            if len(groups) != 1:
                """print("'%s' doesn't have exactly one Group element."
                      " Each group.xml file is allowed to have just one group in it, "
                      "if you want to split a group into two, "
                      "move the other half to a different folder!" % (group_xml_path))
                """
                continue

            for element in tree.findall(".//" + xccdf.XMLNS + "Rule"):
                checks = element.findall(xccdf.XMLNS + "check")
                if len(checks) != 1:
                    print ("Rule of id ", element.get("id", ""),
                           " from ", group_xml_path,
                           " doesn't have exactly one check element!")
                    continue

                check = checks[0]

                if check.get("system") != SCE:
                    print ("Rule of id '", element.get("id", ""),
                           "' from ", group_xml_path, " has system name different from the SCE system name ",
                           "('", SCE, "')!")

                crefs = check.findall(xccdf.XMLNS + "check-content-ref")
                if len(crefs) != 1:
                    print("Rule of id '", element.get("id", ""),
                          "' from '", group_xml_path,
                          "' doesn't have exactly one check-content-ref inside its check element!")
                    continue

                # cref = crefs[0]

                # Check if the description contains a list of affected files
                description = element.find(xccdf.XMLNS + "description")
                if description is None:
                    print ("Rule ", element.get("id", ""), " missing a description")
                    continue

            if b_subgroups:
                cls.perform_autoqa(os.path.join(path_prefix, f), subgroups)

    @classmethod
    def repath_group_xml_tree(cls, source_dir, new_base_dir, group_tree):
        for f, t in six.iteritems(group_tree):
            tree, subgroups = t

            old_base_dir = os.path.join(source_dir, f)

            path_prefix = os.path.relpath(old_base_dir, new_base_dir)
            for element in tree.findall(".//" + xccdf.XMLNS + "check-content-ref"):
                old_href = element.get("href")
                assert(old_href is not None)
                element.set("href", os.path.join(path_prefix, old_href))

            cls.repath_group_xml_tree(old_base_dir, new_base_dir, subgroups)

    @classmethod
    def merge_trees(cls, target_tree, target_element, group_tree):
        def get_sorting_key_for_tree(group_tree, tree_key):
            prefix = 100
            tree, dummy_subgroups = group_tree[tree_key]
            try:
                prefix = int(tree.findall(XCCDF_FRAGMENT + "sort-prefix")[-1].text)
            except:
                pass

            return prefix, tree_key

        for f in sorted(six.iterkeys(group_tree), key=lambda tree_key: get_sorting_key_for_tree(group_tree, tree_key)):
            t = group_tree[f]
            tree, subgroups = t

            groups = tree.findall(xccdf.XMLNS + "Group")
            if len(groups) != 1:
                print("There are %i groups in '%s/group.xml' file. Exactly 1 group is expected! Skipping..." % (len(groups), f))
                continue
            target_element.append(groups[0])
            for child in tree.findall(xccdf.XMLNS + "Profile"):
                assert(child.get("id") is not None)
                merged = False

                # look through profiles in the template XCCDF
                for profile in target_tree.findall(xccdf.XMLNS + "Profile"):
                    if profile.get("id") == child.get("id"):
                        for profile_child in child.findall("*"):
                            profile.append(profile_child)

                        merged = True
                        break

                if not merged:
                    print("Found profile of id '%s' that doesn't match any profiles in template, skipped!" % (child.get("id")), sys.stderr)

            cls.merge_trees(target_tree, groups[0], subgroups)

    @classmethod
    def resolve_selects(cls, target_tree):
        default_selected_rules = set([])
        all_rules = set([])

        for profile in target_tree.findall(xccdf.XMLNS + "Profile"):
            selected_rules = set(default_selected_rules)

            # to avoid invalidating iterators
            to_remove = []
            for select in profile.findall("*"):

                if select.tag == xccdf.XMLNS + "select":
                    if select.get("selected", "false") == "true":
                        selected_rules.add(select.get("idref", ""))
                    else:
                        selected_rules.remove(select.get("idref", ""))

                    to_remove.append(select)

                elif select.tag == XCCDF_FRAGMENT + "meta-select":
                    needle = select.get("idref")
                    for rule_id in all_rules:
                        if re.match(needle, rule_id):
                            if select.get("selected", "false") == "true":
                                selected_rules.add(rule_id)
                            elif rule_id in selected_rules:
                                selected_rules.remove(rule_id)

                    to_remove.append(select)

            for rm in to_remove:
                profile.remove(rm)

            for rule in selected_rules:
                # if it's selected by default, we don't care
                if rule not in default_selected_rules:
                    elem = ElementTree.Element(xccdf.XMLNS + "select")
                    elem.set("idref", rule)
                    elem.set("selected", "true")
                    profile.append(elem)

            for rule in default_selected_rules:
                if rule not in selected_rules:
                    elem = ElementTree.Element(xccdf.XMLNS + "select")
                    elem.set("idref", rule)
                    elem.set("selected", "false")
                    profile.append(elem)

    @classmethod
    def update_content_ref(cls, target_tree, content):
        if not content:
            return target_tree
        for content_ref in target_tree.findall(".//" + xccdf.XMLNS + "check"):
            for check_ref in content_ref.findall("*"):
                if check_ref.tag == xccdf.XMLNS + "check-content-ref":
                    reference = check_ref.get("href")
                    check_ref.set("href", os.path.basename(reference))

        return target_tree

    @classmethod
    def refresh_status(cls, target_tree):
        for status in target_tree.findall(xccdf.XMLNS + "status"):
            if status.get("date", "") == "${CURRENT_DATE}":
                status.set("date", datetime.date.today().strftime("%Y-%m-%d"))

    # taken from http://effbot.org/zone/element-lib.htm#prettyprint
    @classmethod
    def indent(cls, elem, level=0):
        i = "\n" + level*"  "
        if len(elem):
            if not elem.text or not elem.text.strip():
                elem.text = i + "  "
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
            for elem in elem:
                cls.indent(elem, level+1)
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
        else:
            if level and (not elem.tail or not elem.tail.strip()):
                elem.tail = i

    @classmethod
    def get_template_file(cls):
        return os.path.join(os.path.dirname(__file__), "template.xml")

    @classmethod
    def run_compose(cls, target_tree, dir_name, content=None, generate_from_ini=True):
        settings.UPGRADE_PATH = dir_name
        if os.path.exists(os.path.join(dir_name, settings.file_list_rules)):
            os.unlink(os.path.join(dir_name, settings.file_list_rules))
        group_xmls = cls.collect_group_xmls(dir_name, content=content, level=0, generate_from_ini=generate_from_ini)
        cls.perform_autoqa(dir_name, group_xmls)
        new_base_dir = dir_name
        cls.repath_group_xml_tree(dir_name, new_base_dir, group_xmls)
        cls.merge_trees(target_tree, target_tree, group_xmls)
        target_tree = cls.update_content_ref(target_tree, content)
        cls.resolve_selects(target_tree)
        cls.refresh_status(target_tree)
        cls.indent(target_tree)

        return target_tree




from __future__ import unicode_literals
import unittest
import tempfile
import shutil
# import shlex

import os

from preuputils.compose import XCCDFCompose, ComposeXML
from preuputils import variables
from preup import utils
from preup import settings

import base

FOO_DIR = 'FOOBAR6_7'
FOO_RESULTS = FOO_DIR + variables.result_prefix


class TestContentGenerate(base.TestCase):
    def setUp(self):
        self.dir_name = os.path.join(os.getcwd(), 'tests', FOO_DIR, 'dummy')
        self.result_dir = os.path.join(os.getcwd(), 'tests', FOO_RESULTS, 'dummy')

    def tearDown(self):
        shutil.rmtree(os.path.join('tests', FOO_RESULTS))

    def test_compose(self):
        expected_contents = ['failed', 'fixed', 'needs_action', 'needs_inspection', 'not_applicable', 'pass']
        for content in expected_contents:
            compose_xml = ComposeXML()
            result_dir = os.path.join(self.result_dir, content)
            compose_xml.collect_group_xmls(self.dir_name, content=content)
            self.assertTrue(os.path.exists(os.path.join(result_dir, 'group.xml')))
            self.assertFalse(os.path.exists(os.path.join(result_dir, 'all-xccdf.xml')))


class TestGlobalContent(base.TestCase):

    def setUp(self):
        self.temp_dir = tempfile.mktemp(prefix='preupgrade', dir='/tmp')
        self.dir_name = os.path.join(os.getcwd(), 'tests', FOO_DIR)
        self.result_dir = os.path.join(self.temp_dir, FOO_DIR + '-results')
        shutil.copytree(self.dir_name, os.path.join(self.temp_dir, FOO_DIR))

    def tearDown(self):
        shutil.rmtree(self.temp_dir)

    def test_final_compose(self):
        expected_contents = ['failed', 'fixed', 'needs_action', 'needs_inspection', 'not_applicable', 'pass']
        for content in expected_contents:
            compose_xml = ComposeXML()
            dir_name = os.path.join(self.temp_dir, FOO_DIR, 'dummy')
            compose_xml.collect_group_xmls(dir_name, content=content)

        xccdf_compose = XCCDFCompose(os.path.join(self.temp_dir, FOO_DIR))
        xccdf_compose.generate_xml()
        all_xccdf = os.path.join(self.result_dir, settings.content_file)
        self.assertTrue(os.path.exists(all_xccdf))
        dummy_lines = utils.get_file_content(all_xccdf, 'rb')


def suite():
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTest(loader.loadTestsFromTestCase(TestContentGenerate))
    suite.addTest(loader.loadTestsFromTestCase(TestGlobalContent))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=3).run(suite())



from __future__ import print_function, unicode_literals

import sys
import os
import re
import mimetypes

from preup.utils import get_file_content, write_to_file, print_error_msg
from preup import settings
from preup.exception import MissingFileInContentError, MissingHeaderCheckScriptError


def get_full_path(dir_name, script_name):
    """The function returns full path"""
    return os.path.join(dir_name, script_name)


def check_scripts(type_name, dir_name, script_name=None):
    """
    The function checks whether script exists in content directory
    If check_script exists then the script checks whether it is executable
    """
    if not os.path.exists(get_full_path(dir_name, script_name)):
        print ("ERROR: ", dir_name, script_name, "Script name does not exists")
        print ("List of directory (", dir_name, ") is:")
        for file_name in os.listdir(dir_name):
            print (file_name)
        raise MissingFileInContentError
    if type_name != 'solution':
        check_executable(dir_name, script_name)


def apply_function(updates, begin_fnc, end_fnc, dummy_sep, script_type):
    """
    The function generates check_applies_to function into check_scripts
    mentioned in BASH or Python
    """
    template = 'check_applies_to ' + begin_fnc
    if script_type == "sh":
        template += '"' + updates['check_applies'] + '"'
    else:
        template += 'check_applies="' + updates['check_applies'] + '"'
    template += end_fnc
    return template


def rpm_bin_function(updates, begin_fnc, end_fnc, sep, script_type):
    """
    The function generates check_rpm_to function into check_scripts
    mentioned in BASH or Python
    """
    if script_type == "python":
        fnc_name = ['check_rpm="', 'check_bin="']
    else:
        fnc_name = ['"', '"']
    template = 'check_rpm_to ' + begin_fnc + fnc_name[0]

    if "check_rpm" in updates and updates['check_rpm'] is not None:
        template += updates['check_rpm'] + '"'
    else:
        template += '"'
    template += sep + fnc_name[1]
    if "check_bin" in updates and updates['check_bin'] is not None:
        template += updates['check_bin'] + '"'
    else:
        template += '"'
    template += end_fnc

    return template


def generate_common_stuff(lic, updates, script_type):
    """
    Function generates a common stuff
    for Python and BASH scripts
    """
    generated_section = []
    functions = ['check_applies_to', 'check_rpm_to', 'switch_to_content']
    pre_comment = ""
    begin_fnc = ""
    end_fnc = ""
    sep = " "
    if script_type == "sh":
        pre_comment = '#'
    else:
        lic = '"""'+lic+'"""'
        begin_fnc = "("
        end_fnc = ")"
        sep = ","
    if settings.autocomplete:
        for row_lic in lic.split('\n'):
            functions.append(pre_comment + row_lic)
            generated_section.append(pre_comment + row_lic)
    if script_type == "sh":
        functions.append('/usr/share/preupgrade/common.sh')
        generated_section.append('. /usr/share/preupgrade/common.sh')
    if "check_applies" in updates:
        generated_section.append(
            apply_function(updates, begin_fnc, end_fnc, sep, script_type)
        )
    if "check_bin" in updates or "check_rpm" in updates:
        generated_section.append(
            rpm_bin_function(updates, begin_fnc, end_fnc, sep, script_type)
        )
    return generated_section, functions


def update_check_script(dir_name, updates, script_name=None, author=""):
    """
    The function updates check script with license file
    and with API functions like check_rpm_to and check_applies_to
    """
    script_type = get_script_type(dir_name, script_name)
    if author == "":
        author = "<empty_line>"
    generated_section, functions = generate_common_stuff(settings.license.format(author),
                                                         updates,
                                                         script_type)
    full_path_script = get_full_path(dir_name, script_name)
    lines = get_file_content(full_path_script, "rb", method=True)
    if not [x for x in lines if re.search(r'#END GENERATED SECTION', x)]:
        print_error_msg("#END GENERATED SECTION is missing in check_script {0}".format(full_path_script))
        raise MissingHeaderCheckScriptError
    for func in functions:
        lines = [x for x in lines if func not in x.strip()]
    output_text = ""
    for line in lines:
        if '#END GENERATED SECTION' in line:
            new_line = '\n'.join(generated_section)
            new_line = new_line.replace('<empty_line>', '').replace('<new_line>', '')
            output_text += new_line+'\n'
            if 'check_applies' in updates:
                component = updates['check_applies']
            else:
                component = "distribution"
            if script_type == "sh":
                output_text += 'COMPONENT="'+component+'"\n'
            else:
                output_text += 'set_component("'+component+'")\n'
        output_text += line
    write_to_file(full_path_script, "wb", output_text)


def check_executable(dir_name, script_name=""):
    """
    The function checks whether script is executable.
    If not then ERROR message arise
    """
    if not os.access(get_full_path(dir_name, script_name), os.X_OK):
        print_error_msg(title="The file %s is not executable" % os.path.join(dir_name, script_name))


def get_script_type(dir_name, script_name=""):
    """
    The function returns type of check_script.
    If it's not any script then return just txt
    """
    mime_type = mimetypes.guess_type(get_full_path(dir_name, script_name))[0]
    if mime_type is None:
        # try get mime type with shebang
        line = get_file_content(get_full_path(dir_name, script_name), "rb", True)[0]
        if line.startswith("#!"):
            if re.search(r"\bpython[0-9.-]*\b", line):
                return 'python'
            if re.search(r"\bperl[0-9.-]*\b", line):
                return 'perl'
            if re.search(r"\bcsh\b", line):
                return 'csh'
            if re.search(r"\b(bash|sh)\b", line):
                return 'sh'
    file_types = {'text/x-python': 'python',
                  'application/x-csh': 'csh',
                  'application/x-sh': 'sh',
                  'application/x-perl': 'perl',
                  'text/plain': 'txt',
                  None: 'txt',
                  }
    return file_types[mime_type]


def check_inplace_risk(dir_name, prefix="", script_name="", check_func=None):
    """
    The function checks inplace risks
    in check_script and informs user in case of wrong usage
    """
    if check_func is None:
        check_func = []
    lines = get_file_content(get_full_path(dir_name, script_name), "rb")
    compile_req = re.compile(r'^#', re.M|re.I)
    lines = [x for x in lines if not compile_req.search(x.strip())]
    inplace_lines = [x for x in lines if prefix in x]
    for line in inplace_lines:
        tags = [x for x in check_func if re.findall('\\b'+x.split()[0]+'\\b', line)]
        if not tags:
            continue
        checks = [x for x in check_func if len(x.split()) > 1]
        tags = [x for x in checks if re.findall('\\b'+x.split()[1]+'\\b', line)]
        if not tags:
            continue



# -*- coding: utf-8 -*-

from __future__ import print_function, unicode_literals
#import argparse
import optparse
from optparse import OptionValueError

from preup.constants import *


def upload_callback(option, dummy_opt_str, dummy_value, parser):
    if len(parser.rargs) == 0:
        setattr(parser.values, option.dest, True)
    else:
        if parser.rargs[0].startswith('-'):
            setattr(parser.values, option.dest, True)
        else:
            setattr(parser.values, option.dest, parser.rargs[0])
            try:
                second_arg = parser.rargs[1]
            except IndexError:
                pass
            else:
                if not second_arg.startswith('-'):
                    raise OptionValueError("Specify at most one argument for upload option.")


def optional_rh_arg(arg_default):
    def func(option, opt_str, value, parser):
        if parser.rargs and not parser.rargs[0].startswith('-'):
            val = parser.rargs[0]
            parser.rargs.pop(0)
        else:
            val = arg_default
        setattr(parser.values, option.dest, val)
    return func


class CLI(object):

    """Class for processing data from commandline"""

    def __init__(self, args=None):
        """parse arguments"""
        self.parser = optparse.OptionParser(usage=USAGE, description=PROGRAM_DESCRIPTION)

        #self.parser.usage = "%%prog [-v] <content_file>"

        self.add_args()
        if args:
            self.opts, self.args = self.parser.parse_args(args=args)
        else:
            self.opts, self.args = self.parser.parse_args()

    def add_args(self):
        self.parser.add_option(
            "--skip-common",
            action="store_true",
            help="Skip generation of common files"
        )
        self.parser.add_option(
            "-d", "--debug",
            action="store_true",
            help="Turn on debugging mode"
        )
        self.parser.add_option(
            "-u", "--upload",
            dest="upload",
            action="callback",
            callback=upload_callback,
            #metavar="http://127.0.0.1:8000/submit/",
            help="--upload http://127.0.0.1:8000/submit/\n\n\n\n\n\n\n\n\n\n\n\n\n\
Upload results to preupgrade assistant WEB-UI, \
optionally provide URL (otherwise default UI configuration \
will used -- http://127.0.0.1:8099/submit/)"
        )
        self.parser.add_option(
            "-r", "--results",
            type=str,
            metavar="results.tar.gz",
            help="Path to tarball with results when uploading to WEB-UI"
        )
        self.parser.add_option(
            "-l",
            "--list-contents-set",
            action="store_true",
            help="List upgrade path"
        )
        self.parser.add_option(
            "-s",
            "--scan",
            metavar="PATH",
            help="Assess source system"
        )
        self.parser.add_option(
            "-c",
            "--contents",
            help="Path to contents set"
        )
        self.parser.add_option(
            "--riskcheck",
            action="store_true",
            default=False,
            help="Checks preupgrade assessment for INPLACE RISKS." + "\n"*15 +
                 "Return values:" + "\n"*45 +
                 "0 ... NONE, SLIGHT risks were detected." + "\n" * 20 +
                 "1 ... MEDIUM, HIGH risks were detected." + "\n" * 20 +
                 "2 ... EXTREME risk was detected."
        )
        self.parser.add_option(
            "--force",
            action="store_true",
            default=False,
            help="Suppress user interaction"
        )
        self.parser.add_option(
            "--text",
            action="store_true",
            default=False,
            help="Convert HTML results to text form by elinks, lynx or w3m"
        )
        self.parser.add_option(
            "-v",
            "--verbose",
            action="count",
            default=0,
            help="Verbose mode"
        )
        self.parser.add_option(
            "--cleanup",
            action="store_true",
            default=False,
            help="Clean data created by preupgrade-assistant"
        )
        self.parser.add_option(
            "-m",
            "--mode",
            metavar="MODE",
            choices=['migrate', 'upgrade'],
            help="Select mode which can be used for migration or upgrade"
        )
        self.parser.add_option(
            "--kickstart",
            action="store_true",
            default=False,
            help="Generate kickstart"
        )
        self.parser.add_option(
            "--select-rules",
            metavar="RULES",
            help="Run a set of contents give as a comma-separated list."
        )
        self.parser.add_option(
            "--list-rules",
            action="store_true",
            default=False,
            help="List available contents"
        )
        self.parser.add_option(
            "--version",
            action="store_true",
            default=False,
            help="Show Preupgrade Assistant version"
        )

if __name__ == '__main__':
    x = CLI()




from __future__ import unicode_literals
import unittest
import os
import shutil
import tempfile

from preuputils import variables
from preup.application import Application
from preup.conf import Conf, DummyConf
from preup.cli import CLI
from preup import settings, utils, xccdf
from preup.report_parser import ReportParser
from xml.etree import ElementTree
from preuputils.compose import ComposeXML

import base

FOOBAR6_dummy = os.path.join('tests', 'FOOBAR6_7', 'dummy')
FOOBAR6_results = os.path.join('tests', 'FOOBAR6_7' + variables.result_prefix)


def generate_test_xml(path_name, test):
    dir_name = os.path.join(os.getcwd(), path_name)
    prepare_xml(dir_name)
    update_xml(dir_name, test)


def prepare_xml(path_name):
    settings.autocomplete = False
    template_file = ComposeXML.get_template_file()
    tree = None
    try:
        f = open(template_file, "r")
        tree = ElementTree.fromstring(f.read())
        f.close()
    except IOError:
        assert False
    tree = ComposeXML.run_compose(tree,
                                  os.path.dirname(path_name),
                                  content=os.path.basename(path_name))
    try:
        file = open(os.path.join(path_name, 'all-xccdf.xml'), "w")
        file.write(ElementTree.tostring(tree, encoding="utf-8").decode('utf-8'))
        file.close()
    except IOError as e:
        raise


def prepare_cli(temp_dir, path_name):
    conf = {
        "contents": path_name + "/all-xccdf.xml",
        "profile": "xccdf_preupg_profile_default",
        "result_dir": temp_dir,
        "skip_common": True,
        "temp_dir": temp_dir,
        "id": None,
        "quiet": True,
        "debug": True,  # so root check won't fail
    }
    dc = DummyConf(**conf)
    cli = CLI(["--contents", path_name + "/" + settings.xml_result_name])
    a = Application(Conf(dc, settings, cli))
    # Prepare all variables for test
    a.conf.source_dir = os.getcwd()
    a.content = a.conf.contents
    a.basename = os.path.basename(a.content)
    return a


def update_xml(path_name, test):
    dir_name = os.path.join(os.getcwd(), path_name)
    full_path = os.path.join(dir_name, settings.content_file)
    f = open(full_path, mode='r')
    lines = f.readlines()
    f.close()
    lines = [x for x in lines if '<ns0:platform idref="cpe:/o:' not in x]

    for index, line in enumerate(lines):
        if 'tmp_preupgrade' in line:
            lines[index+1] = lines[index+1].replace('SCENARIO', os.path.join(os.getcwd(), 'tests/FOOBAR6_7-results'))
        if 'current_directory' in line:
            lines[index+1] = lines[index+1].replace('SCENARIO', os.path.join(os.getcwd(), 'tests/FOOBAR6_7-results'))
        if 'check-content-ref href="'+test in line:
            lines[index] = line.replace('content-ref href="{0}/'.format(test),
                                        'content-ref href="')

    try:
        f = open(full_path, mode='w')
        f.writelines(lines)
    finally:
        f.close()
    utils.update_platform(full_path)


def delete_tmp_xml(path_name):
    if os.path.exists(os.path.join(path_name, "all-xccdf.xml")):
        os.unlink(os.path.join(path_name, "all-xccdf.xml"))
    if os.path.exists(os.path.join(path_name, "group.xml")):
        os.unlink(os.path.join(path_name, "group.xml"))
    if os.path.isdir(path_name):
        shutil.rmtree(path_name)


def get_result_tag(temp_dir):
    content = utils.get_file_content(os.path.join(temp_dir, settings.xml_result_name), 'rb')
    if not content:
        return []
    target_tree = ElementTree.fromstring(content)

    text = ""
    for test_result in target_tree.findall(".//"+xccdf.XMLNS+"rule-result"):
        for result in test_result.findall(xccdf.XMLNS+"result"):
            text = result.text
    return text


class TestOSCAPPass(base.TestCase):

    def setUp(self):
        self.name = 'pass'
        self.temp_dir = tempfile.mkdtemp()
        self.path_name = os.path.join(FOOBAR6_dummy, self.name)
        self.result_name = os.path.join(FOOBAR6_results, 'dummy', self.name)
        delete_tmp_xml(self.result_name)
        shutil.copytree(self.path_name, self.result_name)

    def tearDown(self):
        shutil.rmtree(self.temp_dir)
        delete_tmp_xml(FOOBAR6_results)

    def test_pass(self):
        """
        Basic test for PASS SCE
        """
        generate_test_xml(self.result_name, self.name)
        # Delete platform tags
        test_log = 'test_log'
        a = prepare_cli(self.temp_dir, self.result_name)
        return_string = utils.run_subprocess(' '.join(a.build_command()), shell=True, output=test_log)
        self.assertEqual(return_string, 0)
        lines = utils.get_file_content(test_log, perms='rb')
        os.unlink(test_log)
        self.assertEqual(a.run_scan(), 0)
        value = get_result_tag(self.temp_dir)
        self.assertTrue(value)
        self.assertEqual(value, "pass")


class TestOSCAPFail(base.TestCase):
    def setUp(self):
        self.name = 'failed'
        self.temp_dir = tempfile.mkdtemp()
        self.path_name = os.path.join(FOOBAR6_dummy, self.name)
        self.result_name = os.path.join(FOOBAR6_results, 'dummy', self.name)
        delete_tmp_xml(self.result_name)
        shutil.copytree(self.path_name, self.result_name)

    def tearDown(self):
        shutil.rmtree(self.temp_dir)
        delete_tmp_xml(FOOBAR6_results)

    def test_fail(self):
        """
        Basic test for FAIL SCE
        """
        generate_test_xml(self.result_name, self.name)
        a = prepare_cli(self.temp_dir, self.result_name)
        self.assertEqual(a.run_scan(), 2)
        value = get_result_tag(self.temp_dir)
        self.assertTrue(value)
        self.assertEqual(value, "fail")


class TestOSCAPNeedsInspection(base.TestCase):
    def setUp(self):
        self.name = 'needs_inspection'
        self.temp_dir = tempfile.mkdtemp()
        self.path_name = os.path.join(FOOBAR6_dummy, self.name)
        self.result_name = os.path.join(FOOBAR6_results, 'dummy', self.name)
        delete_tmp_xml(self.result_name)
        shutil.copytree(self.path_name, self.result_name)

    def tearDown(self):
        shutil.rmtree(self.temp_dir)
        delete_tmp_xml(FOOBAR6_results)

    def test_needs_inspection(self):
        """
        Basic test for FAIL SCE
        """
        generate_test_xml(self.result_name, self.name)
        a = prepare_cli(self.temp_dir, self.result_name)
        self.assertEqual(a.run_scan(), 2)
        report_parser = ReportParser(os.path.join(self.temp_dir, settings.xml_result_name))
        report_parser.replace_inplace_risk()
        value = get_result_tag(self.temp_dir)
        self.assertTrue(value)
        self.assertEqual(value, "needs_inspection")


class TestOSCAPNeedsAction(base.TestCase):
    def setUp(self):
        self.name = 'needs_action'
        self.temp_dir = tempfile.mkdtemp()
        self.path_name = os.path.join(FOOBAR6_dummy, self.name)
        self.result_name = os.path.join(FOOBAR6_results, 'dummy', self.name)
        delete_tmp_xml(self.result_name)
        shutil.copytree(self.path_name, self.result_name)

    def tearDown(self):
        shutil.rmtree(self.temp_dir)
        delete_tmp_xml(FOOBAR6_results)

    def test_needs_action(self):
        """
        Basic test for FAIL SCE
        """
        generate_test_xml(self.result_name, self.name)
        a = prepare_cli(self.temp_dir, self.result_name)
        self.assertEqual(a.run_scan(), 2)
        report_parser = ReportParser(os.path.join(self.temp_dir, settings.xml_result_name))
        report_parser.replace_inplace_risk()
        value = get_result_tag(self.temp_dir)
        self.assertTrue(value)
        self.assertEqual(value, "needs_action")


class TestOSCAPNotApplicable(base.TestCase):
    def setUp(self):
        self.name = 'not_applicable'
        self.temp_dir = tempfile.mkdtemp()
        self.path_name = os.path.join(FOOBAR6_dummy, self.name)
        self.result_name = os.path.join(FOOBAR6_results, 'dummy', self.name)
        delete_tmp_xml(self.result_name)
        shutil.copytree(self.path_name, self.result_name)

    def tearDown(self):
        shutil.rmtree(self.temp_dir)
        delete_tmp_xml(FOOBAR6_results)

    def test_not_applicable(self):
        """
        Basic test for NOT_APPLICABLE SCE
        """
        generate_test_xml(self.result_name, self.name)
        a = prepare_cli(self.temp_dir, self.result_name)
        self.assertEqual(a.run_scan(), 0)
        value = get_result_tag(self.temp_dir)
        self.assertTrue(value)
        self.assertEqual(value, "notapplicable")


class TestOSCAPFixed(base.TestCase):
    def setUp(self):
        self.name = 'fixed'
        self.temp_dir = tempfile.mkdtemp()
        self.path_name = os.path.join(FOOBAR6_dummy, self.name)
        self.result_name = os.path.join(FOOBAR6_results, 'dummy', self.name)
        delete_tmp_xml(self.result_name)
        shutil.copytree(self.path_name, self.result_name)

    def tearDown(self):
        shutil.rmtree(self.temp_dir)
        delete_tmp_xml(FOOBAR6_results)

    def test_fixed(self):
        """
        Basic test for FIXED SCE
        """
        generate_test_xml(self.result_name, self.name)
        a = prepare_cli(self.temp_dir, self.result_name)
        self.assertEqual(a.run_scan(), 0)
        value = get_result_tag(self.temp_dir)
        self.assertTrue(value)
        self.assertEqual(value, "fixed")


def suite():
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTest(loader.loadTestsFromTestCase(TestOSCAPPass))
    suite.addTest(loader.loadTestsFromTestCase(TestOSCAPFail))
    suite.addTest(loader.loadTestsFromTestCase(TestOSCAPNeedsInspection))
    suite.addTest(loader.loadTestsFromTestCase(TestOSCAPNeedsAction))
    suite.addTest(loader.loadTestsFromTestCase(TestOSCAPNotApplicable))
    suite.addTest(loader.loadTestsFromTestCase(TestOSCAPFixed))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=3).run(suite())
